/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

/*
 * %W% %E%
 */

package com.sun.mail.mbox;

import java.io.*;
import javax.mail.*;

public class MboxStore extends Store {

    String user;
    String home;
    Mailbox mb;
    static Flags permFlags;

    static {
	// we support all flags
	permFlags = new Flags();
	permFlags.add(Flags.Flag.SEEN);
	permFlags.add(Flags.Flag.RECENT);
	permFlags.add(Flags.Flag.DELETED);
	permFlags.add(Flags.Flag.FLAGGED);
	permFlags.add(Flags.Flag.ANSWERED);
	permFlags.add(Flags.Flag.DRAFT);
	permFlags.add(Flags.Flag.USER);
    }

    public MboxStore(Session session, URLName url) {
	super(session, url);

	// XXX - handle security exception
	user = System.getProperty("user.name");
	home = System.getProperty("user.home");
	String os = System.getProperty("os.name");
	try {
	    String cl = "com.sun.mail.mbox." + os + "Mailbox";
	    mb = (Mailbox)Class.forName(cl).newInstance();
	} catch (Exception e) {
	    mb = new DefaultMailbox();
	}
    }

    /**
     * since we do not have any authentication
     * to do and we do not want a dialog put up asking the user for a 
     * password we always succeed in connecting.
     */
    protected boolean protocolConnect(String host, int port, String user,
			      String passwd) throws MessagingException {

	// we always succeed in connecting
	// XXX - should we use the user?
	return true;
    }

    protected void setURLName(URLName url) {
	// host, user, password, and file don't matter so we strip them out
	if (url != null && (url.getUsername() != null ||
			    url.getHost() != null ||
			    url.getFile() != null))
	    url = new URLName(url.getProtocol(), null, -1, null, null, null);
	super.setURLName(url);
    }


    public Folder getDefaultFolder() throws MessagingException {
	checkConnected();

	return new MboxFolder(this, null);
    }

    public Folder getFolder(String name) throws MessagingException {
	checkConnected();

	return new MboxFolder(this, name);
    }

    public Folder getFolder(URLName url) throws MessagingException {
	checkConnected();
	return getFolder(url.getFile());
    }

    private void checkConnected() throws MessagingException {
	if (!isConnected())
	    throw new MessagingException("Not connected");
    }

    MailFile getMailFile(String folder) {
	return mb.getMailFile(user, folder);
    }

    Session getSession() {
	return session;
    }
}
