/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

/*
 * %W% %E%
 */

package com.sun.mail.mbox;

import java.io.*;

public class UNIXInbox extends UNIXFolder implements InboxFile {
    String user;

    /*
     * Superclass UNIXFile loads the library containing all the
     * native code and sets the "loaded" flag if successful.
     */

    public UNIXInbox(String user, String name) {
	super(name);
	this.user = user;
    }

    public boolean lock(String mode) {
	if (!loaded)
	    return false;
	if (!maillock(user, 5))
	    return false;
	if (!super.lock(mode)) {
	    mailunlock();
	    return false;
	}
	return true;
    }

    public void unlock() { 
	super.unlock();
	if (loaded)
	    mailunlock();
    }

    public void touchlock() {
	if (loaded)
	    touchlock0();
    }

    private transient RandomAccessFile lockfile; // the user's ~/.Maillock file
    private String lockfileName;	// its name

    public boolean openLock(String mode) {
	if (mode.equals("r"))
	    return true;
	if (lockfileName == null) {
	    String home = System.getProperty("user.home");
	    lockfileName = home + File.separator + ".Maillock";
	}
	try {
	    lockfile = new RandomAccessFile(lockfileName, mode);
	    boolean ret = UNIXFile.lock(lockfile.getFD(), mode);
	    if (!ret)
		closeLock();
	    return ret;
	} catch (IOException ex) {
	}
	return false;
    }

    public void closeLock() {
	if (lockfile == null)
	    return;
	try {
	    lockfile.close();
	} catch (IOException ex) {
	} finally {
	    lockfile = null;
	}
    }

    private native boolean maillock(String user, int retryCount);
    private native void mailunlock();
    private native void touchlock0();
}
