/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.attachments;

import java.util.HashMap;

public class AttachmentUtils {
	
    /**
     * Retrieve the list of attachments id, given the encoded attachment id string
     * @param encodedAttachments Encoded attachment id list. Actually it has the forms ID1;ID2;ID3
     * @return List of attachments ids
     */
    public static String[] decodeAttachmentsIds(String encodedAttachments){
    	if (encodedAttachments == null)
    		return null;
    	String[] attachmentTokens = encodedAttachments.split(";");
    	return attachmentTokens;    	
    }
    
    /**
     * Retrieves the map of <filename, attachment location> given a list of attachments ids.
     * @param attachmentTokens List of attachments ids, usually produced by the method 
     * decodeAttachmentsIds(String encodedAttachments)
     * @return Map of <filename, attachment location>. Attachment location is dependent on
     * the implementation of AttachmentStore: may be filesystem or database.
     */
    public static HashMap<String, String> decodeAttachmentsIds(String[] attachmentTokens){
    	if (attachmentTokens == null)
    		return null;
    	HashMap<String, String> _internalAttachments = new HashMap<String, String>();
    	String[] keyValuePair = null;
    	for (String token : attachmentTokens){
    		keyValuePair = token.split(",");
    		_internalAttachments.put(keyValuePair[0], keyValuePair[1]);
    	}	
    	return _internalAttachments;
    }
    

}
