/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.commands.equinox;

import java.util.List;

import org.eclipse.ebpm.core.registry.ISpagicRegistry;
import org.eclipse.osgi.framework.console.CommandInterpreter;
import org.eclipse.osgi.framework.console.CommandProvider;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.component.ComponentContext;




public class SpagicCommands implements CommandProvider {
	
	public static final String SPAGIC_SERVICE = "SPAGIC_SERVICE";
	public static final String SPAGIC_CONNECTOR = "SPAGIC_CONNECTOR";
	public static final String SPAGIC_DATASOURCE = "SPAGIC_DATASOURCE";
	
	public static final String SPAGIC_TYPE = "SPAGIC_TYPE";
	private ISpagicRegistry spagicRegistry;
	
	private BundleContext context = null;
	protected void activate(ComponentContext componentContext){
		context = componentContext.getBundleContext();
	}
	
	protected void deactivate(ComponentContext componentContext){
		context  = null;
	}
	private void internalSpagicServiceCmd(String filter, CommandInterpreter intp) {
		try{
			ServiceReference[] services = context.getServiceReferences(null, filter);
			if (services != null) {
				int size = services.length;
				if (size > 0) {
					for (int j = 0; j < size; j++) {
						ServiceReference service = services[j];
						
						
						intp.println(service);
						intp.print("  "); //$NON-NLS-1$
						intp.print("Bundle ");
						intp.print(" "); //$NON-NLS-1$
						intp.println(service.getBundle());
						
					
					}
					
					return;
				}
			}
			//intp.println(ConsoleMsg.CONSOLE_NO_REGISTERED_SERVICES_MESSAGE);
			intp.println(" NO SPAGIC SERVICES ");
		}catch (Exception e) {
			intp.printStackTrace(e);
		}
	}

	// Code common to the spagic_connector, spagic_service, spagic_datasource
	private String makeFilter(String filterStart, CommandInterpreter ci) {
		StringBuilder filter = new StringBuilder(filterStart);
		
		boolean first=true;
		for(String next=ci.nextArgument(); next!=null; next=ci.nextArgument()) {
			if(first) {
				filter.append("(|");
				first = false;
			}
			filter.append("(spagic.id=");
			filter.append(next);
			filter.append(")");			
		}
		if(!first) 
			filter.append(")");
		filter.append(")");			
		return filter.toString();
		
	}
	
	/**
	 * Usage: spagic_connector
	 * Usage: spagic_connector <spagic_id>
	 */
	public void _spagic_connector(CommandInterpreter ci) {
		internalSpagicServiceCmd(
			makeFilter("(&(objectClass=org.eclipse.ebpm.core.ISpagicService)(SPAGIC_TYPE=SPAGIC_CONNECTOR)",
						ci), 
			ci);	
	}
		

	/**
	 * Usage: spagic_service 
	 * Usage: spagic_service  <spagic_id>
	 */
	public void _spagic_service (CommandInterpreter ci) {
		internalSpagicServiceCmd(
			makeFilter("(&(objectClass=org.eclipse.ebpm.core.ISpagicService)(SPAGIC_TYPE=SPAGIC_SERVICE)",
					ci),
			ci);
	}
	
	

	public void _spagic_registry(CommandInterpreter intp) {
		List<String> snapshot = spagicRegistry.getSnaphshot();
		intp.println(" ********* SPAGIC_REGISTRY_SNAPSHOT");
		for ( String s : snapshot){
			intp.println("--> " + s);	
		}
	}
	
	/**
	 * Usage: spagic_datasource
	 * Usage: spagic_datasource  <dsId>
	 */
	public void _spagic_datasource (CommandInterpreter ci) {
		internalSpagicServiceCmd(
			makeFilter("(&(objectClass=javax.sql.DataSource)(SPAGIC_TYPE=SPAGIC_DATASOURCE)",
						ci), 
					ci);
	}
	
	/**
	 * Usage: spagic_stop_connector <spagic_id>
	 */
	public void _spagic_stop_connector (CommandInterpreter ci) {
		
	}
	
	/**
	 * Usage: spagic_start_connector <spagic_id>
	 */
	public void _spagic_start_connector (CommandInterpreter ci) {
	}
	
	/**
	 * Usage: spagic_pause_connector <spagic_id>
	 */
	public void _spagic_pause_connector (CommandInterpreter ci) {
	}
	
	/**
	 * Usage: spagic_pause_connector <spagic_id>
	 */
	public void _spagic_components (CommandInterpreter ci) {
	}
	
	/**
	 * Usage: spagic_pause_connector <spagic_id>
	 */
	public void _spagic_pending_deployments (CommandInterpreter ci) {
	}
	
	
	public void _spagic_services(CommandInterpreter ci) {		
	}

	public String getHelp() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("---SPAGIC COMMAND HELP---\n");
		return buffer.toString();
	}
	
	public void bindRegistry(ISpagicRegistry registry){
		spagicRegistry = registry;
	}
	
	public void unbindRegistry(ISpagicRegistry registry){
		spagicRegistry = null;
	}
}