package org.eclipse.ebpm.components.javascript;

import java.io.InputStreamReader;

import javax.script.Bindings;
import javax.script.Compilable;
import javax.script.CompiledScript;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

import org.eclipse.ebpm.core.BaseSpagicService;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.util.resources.IResource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class JavascriptComponent extends BaseSpagicService {
	private ScriptEngine engine;
	private CompiledScript compiledScript;
	protected Logger logger = LoggerFactory.getLogger(JavascriptComponent.class);
	
	public IResource script = null;
	
	
	public void init(){
		try{
			logger.debug("Initializing JavascriptComponent with ID ["+getSpagicId()+"]");
			this.script = propertyConfigurator.getResource("script");
			logger.debug("Script Property Is ["+script+"]");
			this.engine = createScriptEngine();
			logger.debug("Engine Created ["+script+"]");
			if (engine == null)
				throw new RuntimeException("Engine is Null");
			if (engine instanceof Compilable) {
					logger.debug("Engine is IstanceOf Compilable ["+script+"]");
	                Compilable compilable = (Compilable) engine;
	                logger.debug("Compiling Script ["+script+"]");
	                compileScript(compilable);
	                logger.debug("Script Compiled ["+script+"]");
	        }
			
		}catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new RuntimeException(" Cannot instantiate Groovy Component", e);
		}
	}
	
	protected void compileScript(Compilable compilable) throws Exception {
        try{
        	this.compiledScript = compilable.compile(new InputStreamReader(script.openStream()));
        }catch (ScriptException e) {
            throw new Exception("Failed to parse compiledScript. Reason:  " + e, e);
        } 
    }

    protected ScriptEngine createScriptEngine() {
       
    	ScriptEngineManager factory = new ScriptEngineManager();
    	ScriptEngine engine = factory.getEngineByName("JavaScript");
    	return engine;
    }
	
    protected void populateBindings(Bindings bindings, Exchange exchange, Message in, Message out) throws Exception {
        bindings.put("exchange", exchange);
        bindings.put("inMessage", in);
        bindings.put("log", logger);
        bindings.put("outMessage", out);
       
    }
    
	@Override
	public boolean run(Exchange exchange, Message in, Message out)
			throws Exception {
		
		//synchronized (this) {
 		
		logger.info(" JavascriptComponent Component -> Run ["+getSpagicId()+"]"  );
		Bindings bindings = null;
     	try{
     		logger.info(" JavascriptComponent Component -> Before Create Bindings ["+getSpagicId()+"]"  );
     		bindings = engine.createBindings();
     		logger.info(" JavascriptComponent Component -> Populating Bindings  ["+getSpagicId()+"]"  );
     		populateBindings(bindings, exchange, in, out);
     	}catch (Throwable e) {
     		logger.error(" JavascriptComponent Component Error", e);
		}
     	try {
     		long startAt = System.currentTimeMillis();
         	runScript(bindings);
         	long endAt = System.currentTimeMillis();
         	logger.info(" Script["+getSpagicId()+" Script ] |"+ (endAt -startAt) );
         
         	logger.info(" Script Component -> End ["+getSpagicId()+"]");
         	return true;
         	
     	}catch (ScriptException e) {
     		logger.error(" Script Component Error", e);
         	throw new Exception("Failed to run compiledScript. Reason: " + e, e);
     	}
     	
	}
	
	 protected void runScript(Bindings bindings) throws ScriptException {
	        if (compiledScript != null) {
	            compiledScript.eval(bindings);
	        }
	 }
}
