/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.components.xslt;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.dom4j.DocumentHelper;
import org.dom4j.io.DocumentResult;
import org.dom4j.io.DocumentSource;
import org.eclipse.ebpm.core.BaseSpagicService;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.util.resources.IResource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class XsltComponent extends BaseSpagicService {
	
	protected Logger logger = LoggerFactory.getLogger(XsltComponent.class);
	private TransformerFactory transformerFactory;
    
    
    public IResource xslt = null;
	
	
	public void init(){
		try{
			this.xslt = propertyConfigurator.getResource("xslt");
		
			
		}catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new RuntimeException(" Cannot instantiate XsltComponent Component", e);
		}
	}
    
   
    // Properties
    // -------------------------------------------------------------------------
    public TransformerFactory getTransformerFactory() {
    	logger.debug("XSLT Transformer::getTranformerFactory::->start");
        if (transformerFactory == null) {
        	logger.debug("XSLT Transformer::TransformerFactory is null::create one");
            transformerFactory = TransformerFactory.newInstance();
            logger.debug("XSLT Transformer::TransformerFactory created");
        }
        logger.debug("XSLT Transformer::getTransformerFactory::transformerFactory " + ((transformerFactory != null) ? "transformerFactory is not null " : "transformerFactory IS NULL"));
        return transformerFactory;
    }

    public void setTransformerFactory(TransformerFactory transformerFactory) {
        this.transformerFactory = transformerFactory;
    }

    public Source getXsltSource() throws Exception {
    	Source xsltSource;
    	logger.debug("XSLT Transformer::getXsltSource::->start");
      
        	logger.debug("XSLT Transformer::getXsltSource::xsltSource is null create it ");
            // lets create a new one each time
            // as we can only read a stream once
            xsltSource = new StreamSource(xslt.openStream());
            logger.debug("XSLT Transformer::getXsltSource::xsltSource created ");
      
        logger.debug("XSLT Transformer::getXsltSource::xsltSource " + ((xsltSource != null) ? "XSLT Source is not null " : "xsltSource IS NULL"));
        return xsltSource;
    }

    

    

	
	public boolean run(Exchange exchange, Message in, Message out)
			throws Exception {
		try{
			Transformer transformer = getTransformerFactory().newTransformer(getXsltSource());
			org.dom4j.Document inDocument = DocumentHelper.parseText(in.getBodyText());
			DocumentSource source = new DocumentSource(inDocument);
			DocumentResult result = new DocumentResult();
			transformer.transform(source, result);

			// return the transformed document
			org.dom4j.Document transformedDoc = result.getDocument();
			out.setBody(transformedDoc.getRootElement().asXML());
			return true;
		}catch (Throwable e) {
			logger.error(e.getMessage(), e);
			e.fillInStackTrace();
			throw new Exception();
		}
	}

   
   
   

    

    
   
    
    

 

}

