/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.filesystem.adapters;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;

import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;

public class BinaryFileAdapter implements IFileAdapter{
	
	public String getOutputFileName(Exchange exchange, Message in){
		String fileName = null;
		fileName = (String)in.getHeader(IFileAdapter.FILE_NAME_PROPERTY);
		if (fileName == null)
			fileName = (String)exchange.getProperty(IFileAdapter.FILE_NAME_PROPERTY);
		return fileName;
	}

	@Override
	public void writeMessage(Exchange exchange, Message in, BufferedWriter bw) throws Exception  {
		throw new UnsupportedOperationException("Binary Marshaller doesn't support messages writing yet");
//		String body = (String)in.getBody();
//		PrintWriter pw = new PrintWriter(bw);
//		pw.write(body);
//		pw.flush();
	}

	public void readFile(Exchange exchange, Message in, InputStream is, String filePath) throws Exception {
        File polledFile = new File(filePath);
        
        // Put the attachment on a temporary file
        File tempFile = File.createTempFile("spagic-attachment", ".tmp" );

		OutputStream out = new FileOutputStream(tempFile);
		byte buf[] = new byte[1024];
		int len;
		while ((len = is.read(buf)) > 0)
			out.write(buf, 0, len);
		out.close();
		is.close();
        
        DataHandler handler = new DataHandler(new FileDataSource(tempFile));
        
        in.addAttachment(polledFile.getName(), handler);
        in.setHeader(FILE_NAME_PROPERTY, polledFile.getName());
        in.setHeader(FILE_PATH_PROPERTY, filePath);
        
        // Produce a body to avoid an output empty message
        in.setBody("<FILE>" + polledFile.getName() + "</FILE>");	    
	}
	
	@Override
	public String getAdapterId() {
		return "BINARY";
	}
	
}
