/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.ftp;

import java.io.FileFilter;

import org.eclipse.ebpm.connectors.ftp.marshaler.FileMarshaler;
import org.eclipse.ebpm.core.routing.IMessageRouter;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;

public interface IFtpReader {

	public static final String PROPERTY_BINARYMODE = "binaryMode";
	public static final String PROPERTY_USERNAME = "username";
	public static final String PROPERTY_PASSWORD = "password";
	public static final String PROPERTY_HOST = "host";
	public static final String PROPERTY_PORT = "port";
	public static final String PROPERTY_MARSHALER = "marshaler";
	public static final String PROPERTY_DELETEFILE = "deleteFile";
	public static final String PROPERTY_RECURSIVE = "recursive";
	public static final String PROPERTY_CHANGEWORKINGDIRECTORY = "changeWorkingDirectory";
	public static final String PROPERTY_FOLDER = "folder";
	public static final String PROPERTY_STATELESS = "stateless";

	public static final String DEFAULT_MARSHALER = "DEFAULT";

	public String getWorkingPath();
    
    public boolean isRecursive();
    
    public FileFilter getFilter();
    
    public FTPClientPool getClientPool();
    
    public boolean isChangeWorkingDirectory();
    
    public boolean isStateless();
    
    public boolean isDeleteFile();
    	
    public FileMarshaler getMarshaler();   
    
	public Exchange getFtpExchange();
	
	public Message getMessage(Exchange exchange);
    
	public void sendFtpExchange(Exchange exchange);
	
	public IMessageRouter getMessageRouter();
	
}
