/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.jms.endpoints;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;

import org.eclipse.ebpm.connectors.jms.marshaler.JmsConsumerMarshaler;
import org.eclipse.ebpm.core.AbstractSpagicConnector;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class TextListener implements MessageListener {

	protected Logger logger = LoggerFactory.getLogger(JmsConsumerEndpoint.class);
	
	private final AbstractSpagicConnector connector;
	private final JmsConsumerMarshaler marshaler;

	public TextListener(AbstractSpagicConnector connector, JmsConsumerMarshaler marshaler) {
		this.connector = connector;
		this.marshaler = marshaler;
	}

	/**
     * Casts the message to a TextMessage and displays its text.
     *
     * @param message     the incoming message
     */
    public void onMessage(Message message) {

		TextMessage msg = null;
		Exchange exchange = null;

        try {
            if (message instanceof TextMessage) {
            	logger.info("Received message");
                msg = (TextMessage) message;
                
                // Use the marshaler to create the exchange
				exchange = marshaler.createExchange(message);
				
				// Send the exchange
				connector.configureExchangeTarget(exchange);
				connector.send(exchange);                
            } else {
                logger.error("Message of wrong type: " +
                    message.getClass().getName());
            }
        } catch (JMSException e) {
        	logger.error("JMSException in onMessage(): ", e);
        } catch (Throwable t) {
        	logger.error("Exception in onMessage():", t);
        }
    }
}
