/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.jms.marshaler;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringWriter;
import java.util.Map;

import javax.activation.DataHandler;
import javax.jms.Message;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.xml.stream.XMLStreamReader;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.soap.core.MessageImpl;
import org.eclipse.ebpm.soap.core.PhaseInterceptorChain;
import org.eclipse.ebpm.soap.interceptors.mime.AttachmentsInInterceptor;
import org.eclipse.ebpm.soap.interceptors.mime.AttachmentsOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.BodyOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.StaxInInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.StaxOutInterceptor;
import org.eclipse.ebpm.soap.util.DomUtil;
import org.w3c.dom.Document;

public class DefaultProviderMarshaler extends AbstractJmsMarshaler implements
    JmsProviderMarshaler {

    private Map<String, Object> jmsProperties;

    /**
     * @return the jmsProperties
     */
    public Map<String, Object> getJmsProperties() {
        return jmsProperties;
    }

    /**
     * @param jmsProperties the jmsProperties to set
     */
    public void setJmsProperties(Map<String, Object> jmsProperties) {
        this.jmsProperties = jmsProperties;
    }

    public Message createMessage(Exchange exchange, org.eclipse.ebpm.messaging.api.Message in, Session session) throws Exception {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        PhaseInterceptorChain chain = new PhaseInterceptorChain();
        chain.add(new AttachmentsOutInterceptor());
        chain.add(new StaxOutInterceptor());
        chain.add(new BodyOutInterceptor());
        org.eclipse.ebpm.soap.api.Message msg = new MessageImpl();
        
        Document document = DomUtil.parse(new StreamSource(new java.io.StringReader(in.getBodyText())));
        msg.setContent(Document.class, document);
        
        msg.setContent(OutputStream.class, baos);
        for (String attId : in.getAttachments().keySet()) {
            msg.getAttachments().put(attId, (DataHandler) in.getAttachment(attId));
        }
        chain.doIntercept(msg);
        TextMessage text = session.createTextMessage(baos.toString());
        if (msg.get(org.eclipse.ebpm.soap.api.Message.CONTENT_TYPE) != null) {
            text.setStringProperty(CONTENT_TYPE_PROPERTY,
                                   (String) msg.get(org.eclipse.ebpm.soap.api.Message.CONTENT_TYPE));
        }
        if (jmsProperties != null) {
            for (Map.Entry<String, Object> e : jmsProperties.entrySet()) {
                text.setObjectProperty(e.getKey(), e.getValue());
            }
        }
        
        // handle correlation ID
        String correlationId = (String) in.getHeader("org.spagic3.components.jms.correlationId");
        if (correlationId != null) {
        	text.setJMSCorrelationID(correlationId);
        }

        if (isCopyProperties()) {
            copyPropertiesFromNM(in, text);
        }

        return text;
    }

    public void populateMessage(Message message, Exchange exchange, org.eclipse.ebpm.messaging.api.Message normalizedMessage) throws Exception {
        if (message instanceof TextMessage) {
            PhaseInterceptorChain chain = new PhaseInterceptorChain();
            chain.add(new AttachmentsInInterceptor());
            chain.add(new StaxInInterceptor());
            org.eclipse.ebpm.soap.api.Message msg = new MessageImpl();
            msg.setContent(InputStream.class, new ByteArrayInputStream(((TextMessage) message).getText().getBytes()));
            if (message.propertyExists(CONTENT_TYPE_PROPERTY)) {
                msg.put(org.eclipse.ebpm.soap.api.Message.CONTENT_TYPE, message.getStringProperty(CONTENT_TYPE_PROPERTY));
            }
            chain.doIntercept(msg);
            XMLStreamReader xmlReader = msg.getContent(XMLStreamReader.class);
            javax.xml.transform.stax.StAXSource src = new javax.xml.transform.stax.StAXSource(xmlReader);
            
            javax.xml.transform.TransformerFactory tfactory = TransformerFactory.newInstance();
            javax.xml.transform.Transformer xform = tfactory.newTransformer();
            java.io.StringWriter writer = new StringWriter();
            javax.xml.transform.Result result = new javax.xml.transform.stream.StreamResult(writer);
            xform.transform(src, result);

            normalizedMessage.setBody(writer.toString());
            
            for (Map.Entry<String, DataHandler> attachment : msg.getAttachments().entrySet()) {
                normalizedMessage.addAttachment(attachment.getKey(), attachment.getValue());
            }
            if (isCopyProperties()) {
                copyPropertiesFromJMS(message, normalizedMessage);
            }
        } else {
            throw new UnsupportedOperationException("JMS message is not a TextMessage");
        }
    }

}
