/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.ebpm.connectors.mail.utils;

import javax.mail.Authenticator;
import javax.mail.PasswordAuthentication;





/**
 * a data holder class for the connection information
 * 
 * @author lhein  for original sources
 * @author Andrea Zoppello for further modifications
 */
public class MailConnectionConfiguration {
    private String host;
    private String password;
    private String protocol;
    private String username;
    private boolean startTLS = false;
    private String folderName = "INBOX";
    private int port = -1;
       
    public MailConnectionConfiguration(){
    	super();
    }

    public MailConnectionConfiguration(String host, String password,
			String protocol, String username, boolean startTLS,
			String folderName, int port) {
		super();
		this.host = host;
		this.password = password;
		this.protocol = protocol;
		this.username = username;
		this.startTLS = startTLS;
		this.folderName = folderName;
		this.port = port;
	}

	/**
     * @return the host
     */
    public String getHost() {
        return this.host;
    }

    /**
     * @param host the host to set
     */
    public void setHost(String host) {
        this.host = host;
    }

    /**
     * @return the password
     */
    public String getPassword() {
        return this.password;
    }

    /**
     * @param password the password to set
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * @return the protocol
     */
    public String getProtocol() {
        return this.protocol.toLowerCase();
    }

    /**
     * @param protocol the protocol to set
     */
    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }

    /**
     * @return the username
     */
    public String getUsername() {
        return this.username;
    }

    /**
     * @param username the username to set
     */
    public void setUsername(String username) {
        this.username = username;
    }

    /**
     * @return the folderName
     */
    public String getFolderName() {
        return this.folderName;
    }

    /**
     * @param folderName the folderName to set
     */
    public void setFolderName(String folderName) {
        this.folderName = folderName;
    }

    /**
     * @return the port
     */
    public int getPort() {
        return this.port;
    }

    /**
     * @param port the port to set
     */
    public void setPort(int port) {
        this.port = port;
    }

    /**
     * flags the used protocol to be secure or not
     * 
     * @return  true if secure otherwise false
     */
    public boolean isSecureProtocol() {
        return this.protocol.equalsIgnoreCase("smtps") || this.protocol.equalsIgnoreCase("pop3s")
               || this.protocol.equalsIgnoreCase("imaps");
    }

    /**
     * returns an authenticator object for use in sessions
     * 
     * @return  the authenticator object
     */
    public Authenticator getAuthenticator() {
        return new Authenticator() {
            /*
             * (non-Javadoc)
             * 
             * @see javax.mail.Authenticator#getPasswordAuthentication()
             */
            @Override
            protected PasswordAuthentication getPasswordAuthentication() {
                PasswordAuthentication pa = new PasswordAuthentication(getUsername(), getPassword());
                return pa;
            }
        };
    }

	public boolean isStartTLS() {
		return startTLS;
	}

	public void setStartTLS(boolean startTLS) {
		this.startTLS = startTLS;
	}
}
