/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.quartz;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.ebpm.connectors.quartz.adapters.DefaultQuartzAdapter;
import org.eclipse.ebpm.connectors.quartz.adapters.IQuartzAdapter;
import org.eclipse.ebpm.connectors.quartz.support.CronTriggerBean;
import org.eclipse.ebpm.connectors.quartz.support.JobDetailBean;
import org.eclipse.ebpm.connectors.quartz.support.SimpleTriggerBean;
import org.eclipse.ebpm.connectors.quartz.support.SpagicQuartzJob;
import org.eclipse.ebpm.core.AbstractSpagicConnector;
import org.eclipse.ebpm.core.routing.IMessageRouter;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.quartz.Calendar;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.ObjectAlreadyExistsException;
import org.quartz.Scheduler;
import org.quartz.SchedulerFactory;
import org.quartz.Trigger;
import org.quartz.impl.StdSchedulerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class QuartzConnector extends AbstractSpagicConnector{

		protected Logger logger = LoggerFactory.getLogger(QuartzConnector.class);
			
//		private Timer pollingScheduler = null;
//		private TimerTask pollingTask = null;
//		private File directory;
//	    private String extensionFilter;
//	    private boolean deleteFile = true;
//	   
//	    private ConcurrentHashMap<String, InputStream> openedStream = new ConcurrentHashMap<String, InputStream>();
//	    private ConcurrentHashMap<String, File> openedFiles = new ConcurrentHashMap<String, File>();
		
		private Scheduler scheduler;
	    private SchedulerFactory factory;
	    private JobDetail jobDetail;
	    private Map<String, Calendar> calendars;
	    private List<Trigger> triggers =  new ArrayList<Trigger>();
	    private Trigger trigger;
	    private IQuartzAdapter adapter = null;
	    
	    /**
	     * @return the job
	     */
	    public JobDetail getJobDetail() {
	        return jobDetail;
	    }

	    /**
	     * @param job the job to set
	     */
	    public void setJobDetail(JobDetail job) {
	        this.jobDetail = job;
	    }
	    
	    /**
	     * @return the calendar
	     */
	    public Map<String, Calendar> getCalendars() {
	        return calendars;
	    }

	    /**
	     * @param calendar the calendar to set
	     */
	    public void setCalendars(Map<String, Calendar> calendars) {
	        this.calendars = calendars;
	    }
	    
	    /**
	     * @return the triggers
	     */
	    public List<Trigger> getTriggers() {
	        return triggers;
	    }

	    /**
	     * @param triggers the triggers to set
	     */
	    public void setTriggers(List<Trigger> triggers) {
	        this.triggers = triggers;
	    }
	    
	    /**
	     * @return the trigger
	     */
	    public Trigger getTrigger() {
	        return trigger;
	    }

	    /**
	     * @param trigger the trigger to set
	     */
	    public void setTrigger(Trigger trigger) {
	        this.trigger = trigger;
	    }
	    
	    /**
	     * @return the trigger
	     */
	    public IQuartzAdapter getAdapter() {
	        return adapter;
	    }

	    public void setAdapter(IQuartzAdapter adapter) {
	        this.adapter = adapter;
	    }
	    
	    public void init() {
	    	
	    	String triggerType;
			String cronExpression;
			Integer repeatCount;
			Long repeatInterval;
			boolean flgInsertJobDetail;
			String jobName;
			String jobGroup;
			String jobDescription;
	    	
	    	triggerType = propertyConfigurator.getString("triggerType");
	    	flgInsertJobDetail = propertyConfigurator.getBoolean("insertJobDetail", false);
	    	
	    	try
	    	{
			
				if (triggerType.equals("cron"))
		    	{
		    		cronExpression = propertyConfigurator.getString("cronExpression");
		    		
		    		CronTriggerBean cronTrigger =  new CronTriggerBean();
		    		cronTrigger.setCronExpression(cronExpression);
		    		cronTrigger.setName(getSpagicId());
		    		
		    		//setTrigger(cronTrigger);
		    		
		    		triggers.add(cronTrigger);
		    	}
		    	else if (triggerType.equals("simple"))
		    	{
		    		repeatCount = propertyConfigurator.getInteger("repeatCount");
					repeatInterval = propertyConfigurator.getLong("repeatInterval");
					
					// current time
			        long ctime = System.currentTimeMillis(); 
		    		
		    		SimpleTriggerBean simpleTrigger =  new SimpleTriggerBean();
		    		// set the name of the trigger
		    		simpleTrigger.setName(getSpagicId());
		    		// set the number of execution of this job
		    		simpleTrigger.setRepeatCount(repeatCount);
		    		// set the interval, how often the job should run
		    		simpleTrigger.setRepeatInterval(repeatInterval);
		    		// set its start up time
		            simpleTrigger.setStartTime(new Date(ctime));
		    		
		    		//setTrigger(simpleTrigger);
		    		
		    		triggers.add(simpleTrigger);
		    	}
				
				if (flgInsertJobDetail)
				{
					jobName = propertyConfigurator.getString("jobName");
			    	jobGroup = propertyConfigurator.getString("jobGroup");
			    	jobDescription = propertyConfigurator.getString("jobDescription");
			    	
			    	jobDetail = new  JobDetailBean();
			    	jobDetail.setName(jobName);
			    	jobDetail.setGroup(jobGroup);
			    	jobDetail.setDescription(jobDescription);
				}
	    	
		    	if (scheduler == null) {
		            if (factory == null) {
		                factory = new StdSchedulerFactory();
		            }
		            scheduler = factory.getScheduler();
		        }

		    	scheduler.getContext().setAllowsTransientData(true);
		        
		    	//scheduler.getContext().put(getComponentName(), this);
		    	scheduler.getContext().put(this.spagicId, this);
		    	
		    	// create new adapter
		    	adapter = new DefaultQuartzAdapter();
		    	
		    	// Execute the validation
		    	validate();	    	
		    	//
	    	}
	    	catch(Exception ex)
	    	{
	    		logger.error(ex.getMessage());
	    		throw new RuntimeException(ex.getMessage());
	    	}
		}
	   
		public void start() throws Exception {
			super.start();
			
			if (!scheduler.isStarted()) scheduler.start();
			
			jobDetail.getJobDataMap().put(SpagicQuartzJob.COMPONENT_NAME, SpagicQuartzJob.COMPONENT_NAME_PARAM_VALUE);
	        jobDetail.getJobDataMap().put(SpagicQuartzJob.ENDPOINT_NAME, getSpagicId());
	        for (Map.Entry<String, Calendar> e : getCalendars().entrySet()) {
	            scheduler.addCalendar(e.getKey(), e.getValue(), true, true);
	        }
	        scheduler.addJob(getJobDetail(), true);
	        for (Trigger trg : getTriggers()) {
	            boolean triggerExists = scheduler.getTrigger(trg.getName(), trg.getGroup()) != null;
	            if (!triggerExists) {
	                try {
	                    scheduler.scheduleJob(trg);
	                } catch (ObjectAlreadyExistsException ex) {
	                    scheduler.rescheduleJob(trg.getName(), trg.getGroup(), trg);
	                }
	            } else {
	                scheduler.rescheduleJob(trg.getName(), trg.getGroup(), trg);
	            }
	        }
			
		}
		
		public void stop() throws Exception {
			super.stop();
			for (Trigger trg : getTriggers()) {
	            scheduler.unscheduleJob(trg.getName(), trg.getGroup());
	        }
	        scheduler.deleteJob(getJobDetail().getName(), getJobDetail().getGroup());
	        for (Map.Entry<String, Calendar> e : getCalendars().entrySet()) {
	            scheduler.deleteCalendar(e.getKey());
	        }
			
		}
		
		public void process(Exchange exchange) {
			//logger.info("Quartz connector process : " + exchange.getIn());
			
		}
		
		
		//public void validate() throws DeploymentException {
		public void validate() throws Exception {
	        //super.validate();
	        //if (trigger instanceof Job  JobDetailAwareTrigger) {
//			if (trigger != null) {
//	            if (jobDetail != null) {
//	                throw new Exception("jobDetail can not be set on endpoint and trigger at the same time");
//	            }
//	            jobDetail = trigger.g ((JobDetailAwareTrigger) trigger).getJobDetail();
//	        }
	        if (jobDetail == null) {
	            JobDetailBean j = new JobDetailBean();
	            //j.setName(EndpointSupport.getKey(this));
	            j.setName(getSpagicId());
		    	j.setDescription(getSpagicId());
	            jobDetail = j;
	        }
	        if (triggers == null) {
	            triggers = new ArrayList<Trigger>();
	        }
	        if (trigger != null && triggers != null && triggers.size() > 0) {
	            throw new Exception("trigger and triggers can not be set at the same time");
	        }
	        if (trigger != null) {
	            triggers.add(trigger);
	        }
	        if (calendars == null) {
	            calendars = new HashMap<String, Calendar>();
	        }
	        for (Trigger t : triggers) {
	            if (t.getCalendarName() != null && calendars.get(t.getCalendarName()) == null) {
	                throw new Exception("Trigger references an unknown calendar " + t.getCalendarName());
	            }
	            t.setJobName(jobDetail.getName());
	            t.setJobGroup(jobDetail.getGroup());
	        }
	    }
		
		public void onJobExecute(JobExecutionContext context) throws JobExecutionException {
	        if (logger.isDebugEnabled()) {
	            logger.debug("Firing Quartz Job with context: " + context);
	        }
	        try {
	            //InOnly exchange = getExchangeFactory().createInOnlyExchange();
	            Exchange exchange = createInOnlyExchange();
	            //NormalizedMessage message = exchange.createMessage();
	            Message inMessage = exchange.getIn(true);
	            getAdapter().populateNormalizedMessage(inMessage, context);
	            //exchange.setInMessage(message);
	            exchange.setIn(inMessage);
	            
	            IMessageRouter router = getMessageRouter();

	            if (!router.isServiceRegistered(this.target)){
	            	logger.warn("Target Service ["+this.target+"] is not Registere cannot fire message");
	            }else{
	            	send(exchange);
	            }
	        } catch (Exception e) {
	        	logger.error("Error in Exection of job " + e.getMessage(), e);
	        }
	    }
		
}
