/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.ebpm.connectors.quartz.support;

import org.eclipse.ebpm.connectors.quartz.QuartzConnector;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.SchedulerException;

/**
 * A Quartz Job which dispatches a message to a component.
 *
 * @version $Revision: 526407 $
 */
public class SpagicQuartzJob implements Job {
    
    public static final String COMPONENT_NAME = "org.spagic3.quartz.ComponentName";
    public static final String ENDPOINT_NAME = "org.spagic3.quartz.EndpointName";
    public static final String COMPONENT_NAME_PARAM_VALUE = "spagic3.component.quartz";
    
    public void execute(JobExecutionContext context) throws JobExecutionException {
        try {        	
        	
        	String connectorId = (String) context.getJobDetail().getJobDataMap().get(SpagicQuartzJob.ENDPOINT_NAME);
            if (connectorId == null) {
                throw new JobExecutionException("No property '" + SpagicQuartzJob.ENDPOINT_NAME + "' defined. Bad job data map");
            }
            QuartzConnector connector = (QuartzConnector) context.getScheduler().getContext().get(connectorId);
            if (connector == null) {
                throw new JobExecutionException("No quartz connector available for key: " + connectorId + "."
                        + " Bad job data map");
            }

            connector.onJobExecute(context);
        } catch (SchedulerException e) {
            throw new JobExecutionException(e);
        }
    }
    
}
