/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.core;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.messaging.api.Status;
import org.eclipse.ebpm.messaging.core.MessageImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class BaseSpagicService extends AbstractSpagicService {

	protected Logger logger = LoggerFactory.getLogger(BaseSpagicService.class);

	private boolean copyAttachments = true;
	private boolean copyProperties = true;

	public boolean isCopyAttachments() {
		return copyAttachments;
	}

	public void setCopyAttachments(boolean copyAttachments) {
		this.copyAttachments = copyAttachments;
	}

	public boolean isCopyProperties() {
		return copyProperties;
	}

	public void setCopyProperties(boolean copyProperties) {
		this.copyProperties = copyProperties;
	}



	public void process(Exchange exchange){

		// If the service is stopped throw an Exception, allowing the caller
		// to perform appropriate actions
		try {
			if( isStopped() ) {
				fail(exchange, new UnsupportedOperationException("Trying to invoke a stopped component. Component Id: " + getSpagicId()));
				return;
			}
		} 
		catch (Exception whatTheHell) {
			// This Exception is in the contract, but nothing
			// should throw it...
			logger.error("Complettly unexpected exception checking service status", whatTheHell);
			throw new RuntimeException(whatTheHell);
		}
		  // Skip done exchanges
        if (exchange.getStatus() == Status.Done) {
            return;
        // Handle error exchanges
        } else if (exchange.getStatus() == Status.Error) {
            return;
        }
        try {
            Exchange outExchange = null;
            Message in = exchange.getIn();
            Message out;
            if (ExchangeUtils.isInAndOut(exchange)) {
                out = exchange.getOut(true);
            } else {
                outExchange = createInOnlyExchange();
                outExchange.setProperty(SpagicConstants.SPAGIC_SENDER, getSpagicId());
                String processCorrelationId = (String)exchange.getProperty(SpagicConstants.CORRELATION_ID);
                if (processCorrelationId != null) {
                    outExchange.setProperty(SpagicConstants.CORRELATION_ID, processCorrelationId);
                }
                out = exchange.getIn(true);
            }
            
            copyPropertiesAndAttachments(exchange, in, out);
            
            if (run(exchange, in, out)) {
                if (ExchangeUtils.isInAndOut(exchange)) {
                
                	configureForResponse(exchange);
                	exchange.setOut(out);
                    send(exchange);
                } else {
                	// *******************************************************
                    // 1 - Send the Out Exchange
                	// *******************************************************
                	outExchange.setIn(out);
                    send(outExchange);
                    
                    // *******************************************************
                    // 2 - Send Back the original Exchange with status DONE
                	// *******************************************************
                    done(exchange);
                }
            } else {
                done(exchange);
            }
        } catch (Exception e) {
                fail(exchange, e);
        }
	}



	 
	 protected void copyPropertiesAndAttachments(Exchange exchange, Message in, Message out) throws Exception {
		 if (isCopyProperties()) {
			out.setHeaders(in.getHeaders());
		 }
		 if (isCopyAttachments()) {
			 String attachmentsIn = ((MessageImpl)in)._encodeAttachments();
			 ((MessageImpl)out)._decodeAttachments(attachmentsIn);
		 }
	 }
	 
	 public boolean run(Exchange exchange, Message in, Message out) throws Exception{
		 return false;
	 }

}
