package org.eclipse.ebpm.dynapp;

import java.io.File;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;

import org.dom4j.Element;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.dirwatcher.DirWatcher;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ApplicationHandler implements BundleListener {
	
	protected Logger logger = LoggerFactory.getLogger(ApplicationHandler.class);
	
	private String applicationId = null;
	private String applicationName = null;
	private String path = null;
	private boolean absolute = false;
	private String spagicHome = null;
	private BundleContext context = null;
	private Timer bundleTimerService = null;
	
	private ConcurrentHashMap<String, Bundle> bundles = new ConcurrentHashMap<String, Bundle>();
	
	//private CopyOnWriteArrayList<Bundle> unresolvedBundles= new CopyOnWriteArrayList<Bundle>();
	public boolean isAbsolute() {
		return absolute;
	}
	public void setAbsolute(boolean absolute) {
		this.absolute = absolute;
	}

	private String spagicHomeLocation = null;
	
	public ApplicationHandler(BundleContext context, String spagicHome, Element el){
		this.context = context;
		this.applicationId = el.attributeValue("id");
		this.applicationName = el.attributeValue("name");
		this.path = el.attributeValue("path");
		this.spagicHome = spagicHome;
		
		String pathType = el.attributeValue("type");
		
		if ( pathType.equalsIgnoreCase("absolute") ){
			setAbsolute(true);
		}
		
	}
	public String getApplicationId() {
		return applicationId;
	}
	public void setApplicationId(String applicationId) {
		this.applicationId = applicationId;
	}
	public String getApplicationName() {
		return applicationName;
	}
	public void setApplicationName(String applicationName) {
		this.applicationName = applicationName;
	}
	public String getPath() {
		return path;
	}
	public void setPath(String path) {
		this.path = path;
	}

	public String getSpagicHomeLocation() {
		return spagicHomeLocation;
	}
	
	public void setSpagicHomeLocation(String spagicHomeLocation) {
		this.spagicHomeLocation = spagicHomeLocation;
	}
	
	public String getApplicationBundlesHome(){
		if (isAbsolute()){
			return path + File.separator + "bundles";
		}else{
			return this.spagicHome + File.separator + path + File.separator + "bundles";
		}
	}
	
	public void start() {
		context.addBundleListener(this);
		final ApplicationHandler appHandler = this;
		TimerTask bundlesPollingTask = 
			new DirWatcher(getApplicationBundlesHome(), SpagicConstants.JAR_EXTENSIONS ) {
			protected void onChange(File file, String action) {
				logger.debug("ApplicationHandler["+applicationId+"]::Bundles Thread -> File ["+file.getPath()+"] has been ["+action+"]");
				if (action.equalsIgnoreCase("delete")){
					logger.debug("ApplicationHandler["+applicationId+"]::Bundles Thread -> File ["+file.getPath()+"] Uninstalling BUNDLE");
					appHandler.uninstallBundle(file);
				}else if (action.equalsIgnoreCase("add")){
					logger.debug("ApplicationHandler["+applicationId+"]::Bundles Thread -> File ["+file.getPath()+"] Installing BUNDLE");
					appHandler.installBundle(file);
				}else if (action.equalsIgnoreCase("modify")){
					logger.debug("ApplicationHandler["+applicationId+"]::Bundles Thread -> File ["+file.getPath()+"] Refreshing BUNDLE");
					appHandler.refreshBundle(file);
				}
			}
		};
		
		bundleTimerService = new Timer();
		bundleTimerService.schedule(bundlesPollingTask, new Date(), 5000);
	}

	public void installBundle(File f){
		Bundle bundle = null;
		try{
			String bundleURI = f.toURI().toURL().toString();
			bundle = context.installBundle(bundleURI);
			this.bundles.put(bundleURI, bundle);
			bundle.start();
		
		}catch (Exception e) {
			
			e.printStackTrace();
		}
	}
	
	public void uninstallBundle(File f){
		
		try{
			String bundleURI = f.toURI().toURL().toString();
			
			Bundle b = bundles.get(bundleURI);
		
			if (b != null){
				b.stop();
				b.uninstall();
			
				bundles.remove(bundleURI);
			}
		}catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	public void refreshBundle(File f){
		uninstallBundle(f);
		installBundle(f);
	}
	
	public void stop() {
		context.removeBundleListener(this);
		try{
			bundleTimerService.cancel();
		
			for ( Bundle b : bundles.values()){
				b.stop();
				b.uninstall();
			}
		}catch (Exception e) {
			e.printStackTrace();
		}
		bundles.clear();
	}
	
	@Override
	public void bundleChanged(BundleEvent event) {
		
		
		if ((event.getType() == BundleEvent.RESOLVED)){
			for (Bundle b : bundles.values()){
				try{
					if (b.getState() != Bundle.ACTIVE){
						b.start();
						
					}
				}catch (Exception e) {
					e.printStackTrace();
				}
			}
			
		}
	}
	
}
