/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.graph.util;

public class GraphNode {
	
	private String id;
	private String title;
	private String URL;
	private String comment;
	private String tooltip = "Detail";
	private int fontSize = 6;
	private String fontName = "Tahoma";
	private String shape = "box";
	private String color = "gray";
	private String style = "rounded";
	private String fillColor = null;
	
	public GraphNode(String id, String title) {
		super();
		this.id = id;
		this.title = title;
		setFillColor("white");
	}
	
	public GraphNode(String id, String title, String url, String comment, String tooltip) {
		super();
		this.id = id;
		this.title = title;
		URL = url;
		this.comment = comment;
		this.tooltip = tooltip;
		setFillColor("white");
	}
	
	public String getComment() {
		return comment;
	}
	public void setComment(String comment) {
		this.comment = comment;
	}
	public String getId() {
		return id;
	}
	public void setId(String id) {
		this.id = id;
	}
	public String getTitle() {
		return title;
	}
	public void setTitle(String title) {
		this.title = title;
	}
	public String getTooltip() {
		return tooltip;
	}
	public void setTooltip(String tooltip) {
		this.tooltip = tooltip;
	}
	public String getURL() {
		return URL;
	}
	public void setURL(String url) {
		URL = url;
	}
	
	public String toDot(){
		
		StringBuffer dotStringBuffer = new StringBuffer();
		dotStringBuffer.append(id + " [label=\""+title+"\"");
		dotStringBuffer.append(", fontname=\""+fontName+"\"");
		dotStringBuffer.append(", fontsize=\""+fontSize+"\"");
		dotStringBuffer.append(", tooltip=\""+tooltip+"\"");
		dotStringBuffer.append(", URL=\""+URL+"\""); 
		dotStringBuffer.append(", color="+color);
		dotStringBuffer.append(", shape="+shape);
		if (fillColor != null){
			dotStringBuffer.append(", fillcolor="+fillColor);
		}
		if (style != null)
			dotStringBuffer.append(", style="+style);
		dotStringBuffer.append(" ]");
		
	
		return dotStringBuffer.toString();
	}

	public int getFontSize() {
		return fontSize;
	}

	public void setFontSize(int fontSize) {
		this.fontSize = fontSize;
	}

	public String getColor() {
		return color;
	}

	public void setColor(String color) {
		this.color = color;
	}

	public String getShape() {
		return shape;
	}

	public void setShape(String shape) {
		this.shape = shape;
	}

	public String getFontName() {
		return fontName;
	}

	public void setFontName(String fontName) {
		this.fontName = fontName;
	}

	public String getStyle() {
		return style;
	}

	public void setStyle(String style) {
		this.style = style;
	}

	public String getFillColor() {
		return fillColor;
	}

	public void setFillColor(String fillColor) {
		this.fillColor = fillColor;
		if (style != null){
			String oldStyle = getStyle();
			if (oldStyle.startsWith("\"")){
				oldStyle = oldStyle.substring(0, oldStyle.length() - 1);
				oldStyle += ",filled\"";
				setStyle(oldStyle);
			}else{
				setStyle("\""+style+",filled\"");
			}
		}else{
			setStyle("filled");
		}
	}
	
}
