/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.ebpm.messaging.api;

import java.util.HashMap;
import java.util.Map;


public enum Pattern {

    InOnly,
    RobustInOnly,
    InOut,
    InOptionalOut;


    protected static final Map<String, Pattern> map;

    /**
     * Returns the WSDL URI for this message exchange pattern
     *
     * @return the WSDL URI for this message exchange pattern
     */
    public String getWsdlUri() {
        switch (this) {
            case InOnly:
                return "http://www.w3.org/ns/wsdl/in-only";
            case InOptionalOut:
                return "http://www.w3.org/ns/wsdl/in-opt-out";
            case InOut:
                return "http://www.w3.org/ns/wsdl/in-out";
            case RobustInOnly:
                return "http://www.w3.org/ns/wsdl/robust-in-only";
            default:
                throw new IllegalArgumentException("Unknown message exchange pattern: " + this);
        }
    }

    /**
     * Converts the WSDL URI into a {@link Pattern} instance
     */
    public static Pattern fromWsdlUri(String wsdlUri) {
        return map.get(wsdlUri);
    }

    static {
        map = new HashMap<String, Pattern>();
        for (Pattern mep : values()) {
            String uri = mep.getWsdlUri();
            map.put(uri, mep);
            String name = uri.substring(uri.lastIndexOf('/') + 1);
            map.put("http://www.w3.org/2004/08/wsdl/" + name, mep);
            map.put("http://www.w3.org/2006/01/wsdl/" + name, mep);
        }
    }
}