/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;
import java.util.List;


/**
 * The persistent class for the pool_type database table.
 * 
 */
@Entity
@Table(name="pool_type")
public class PoolType implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_pool_type_gen")
	@Column(name="id_pool_type")
	@SequenceGenerator(name = "id_pool_type_gen",sequenceName="seq_pool_type", allocationSize = 1)
	private Long idPoolType;

	private String description;

	private String name;

	@Column(name="pool_type_version")
	private String poolTypeVersion;

	//bi-directional many-to-many association to Attribute
    @ManyToMany
	@JoinTable(
		name="attribute_pool"
		, joinColumns={
			@JoinColumn(name="id_pool_type")
			}
		, inverseJoinColumns={
			@JoinColumn(name="id_attribute")
			}
		)
	private List<Attribute> attributes;

	//bi-directional many-to-one association to PoolInstance
	@OneToMany(mappedBy="poolType")
	private List<PoolInstance> poolInstances;

	//bi-directional many-to-one association to Rule
    @ManyToOne
	@JoinColumn(name="id_rule_det_pool_type")
	private Rule rule;

	//bi-directional many-to-one association to PoolTypeProcess
	@OneToMany(mappedBy="poolType")
	private List<PoolTypeProcess> poolTypeProcesses;

    public PoolType() {
    }

	public Long getIdPoolType() {
		return this.idPoolType;
	}

	public void setIdPoolType(Long idPoolType) {
		this.idPoolType = idPoolType;
	}

	public String getDescription() {
		return this.description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getPoolTypeVersion() {
		return this.poolTypeVersion;
	}

	public void setPoolTypeVersion(String poolTypeVersion) {
		this.poolTypeVersion = poolTypeVersion;
	}

	public List<Attribute> getAttributes() {
		return this.attributes;
	}

	public void setAttributes(List<Attribute> attributes) {
		this.attributes = attributes;
	}
	
	public List<PoolInstance> getPoolInstances() {
		return this.poolInstances;
	}

	public void setPoolInstances(List<PoolInstance> poolInstances) {
		this.poolInstances = poolInstances;
	}
	
	public Rule getRule() {
		return this.rule;
	}

	public void setRule(Rule rule) {
		this.rule = rule;
	}
	
	public List<PoolTypeProcess> getPoolTypeProcesses() {
		return this.poolTypeProcesses;
	}

	public void setPoolTypeProcesses(List<PoolTypeProcess> poolTypeProcesses) {
		this.poolTypeProcesses = poolTypeProcesses;
	}
	
}