/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;
import java.util.List;


/**
 * The persistent class for the relevant_data database table.
 * 
 */
@Entity
@Table(name="relevant_data")
public class RelevantData implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_relevant_data_gen")
	@Column(name="id_relevant_data")
	@SequenceGenerator(name = "id_relevant_data_gen", sequenceName="seq_relevant_data", allocationSize = 1)
	private Long idRelevantData;

	private String value;

	//bi-directional many-to-one association to Attribute
    @ManyToOne
	@JoinColumn(name="id_attribute")
	private Attribute attribute;

	//bi-directional many-to-many association to ProcessInstance
    @ManyToMany
	@JoinTable(
		name="relevant_data_process_instance"
		, joinColumns={
			@JoinColumn(name="id_relevant_data")
			}
		, inverseJoinColumns={
			@JoinColumn(name="id_process_instance")
			}
		)
	private List<ProcessInstance> processInstances;

    public RelevantData() {
    }

	public Long getIdRelevantData() {
		return this.idRelevantData;
	}

	public void setIdRelevantData(Long idRelevantData) {
		this.idRelevantData = idRelevantData;
	}

	public String getValue() {
		return this.value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public Attribute getAttribute() {
		return this.attribute;
	}

	public void setAttribute(Attribute attribute) {
		this.attribute = attribute;
	}
	
	public List<ProcessInstance> getProcessInstances() {
		return this.processInstances;
	}

	public void setProcessInstances(List<ProcessInstance> processInstances) {
		this.processInstances = processInstances;
	}
	
}