/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;
import java.math.BigInteger;
import java.util.Date;
import java.util.List;


/**
 * The persistent class for the rule database table.
 * 
 */
@Entity
@Table(name="rule")
public class Rule implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_rule_gen")
	@Column(name="id_rule")
	@SequenceGenerator(name = "id_rule_gen", sequenceName="seq_rule", allocationSize = 1)
	private Long idRule;

	@Column(name="condition_expr")
	private String conditionExpr;

    @Temporal( TemporalType.DATE)
	@Column(name="end_date")
	private Date endDate;

	private String expr;

	private String name;

	@Column(name="rule_type")
	private BigInteger ruleType;

	@Column(name="rule_usage")
	private int ruleUsage;

    @Temporal( TemporalType.DATE)
	@Column(name="start_date")
	private Date startDate;

	@Column(name="transition_state")
	private int transitionState;

	@Column(name="variable_name")
	private String variableName;

	//bi-directional many-to-one association to PoolType
	@OneToMany(mappedBy="rule")
	private List<PoolType> poolTypes;

	//bi-directional many-to-one association to Attribute
    @ManyToOne
	@JoinColumn(name="id_attribute")
	private Attribute attribute;

	//bi-directional many-to-one association to StepRule
	@OneToMany(mappedBy="rule")
	private List<StepRule> stepRules;

    public Rule() {
    }

	public Long getIdRule() {
		return this.idRule;
	}

	public void setIdRule(Long idRule) {
		this.idRule = idRule;
	}

	public String getConditionExpr() {
		return this.conditionExpr;
	}

	public void setConditionExpr(String conditionExpr) {
		this.conditionExpr = conditionExpr;
	}

	public Date getEndDate() {
		return this.endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

	public String getExpr() {
		return this.expr;
	}

	public void setExpr(String expr) {
		this.expr = expr;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public BigInteger getRuleType() {
		return this.ruleType;
	}

	public void setRuleType(BigInteger ruleType) {
		this.ruleType = ruleType;
	}

	public int getRuleUsage() {
		return this.ruleUsage;
	}

	public void setRuleUsage(int ruleUsage) {
		this.ruleUsage = ruleUsage;
	}

	public Date getStartDate() {
		return this.startDate;
	}

	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}

	public int getTransitionState() {
		return this.transitionState;
	}

	public void setTransitionState(int transitionState) {
		this.transitionState = transitionState;
	}

	public String getVariableName() {
		return this.variableName;
	}

	public void setVariableName(String variableName) {
		this.variableName = variableName;
	}

	public List<PoolType> getPoolTypes() {
		return this.poolTypes;
	}

	public void setPoolTypes(List<PoolType> poolTypes) {
		this.poolTypes = poolTypes;
	}
	
	public Attribute getAttribute() {
		return this.attribute;
	}

	public void setAttribute(Attribute attribute) {
		this.attribute = attribute;
	}
	
	public List<StepRule> getStepRules() {
		return this.stepRules;
	}

	public void setStepRules(List<StepRule> stepRules) {
		this.stepRules = stepRules;
	}
	
}