/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;
import java.math.BigInteger;
import java.sql.Timestamp;
import java.util.List;


/**
 * The persistent class for the transition_instance database table.
 * 
 */
@Entity
@Table(name="transition_instance")
public class TransitionInstance implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_journal_gen")
	@Column(name="id_journal")
	@SequenceGenerator(name = "id_journal_gen", sequenceName="seq_transition_instance", allocationSize = 1)
	private Long idJournal;

    @Lob()
	@Column(name="attachment_id")
	private String attachmentId;

	@Column(name="attachment_in")
	private int attachmentIn;

	@Column(name="id_ext_process_instance")
	private BigInteger idExtProcessInstance;

	@Column(name="id_message")
	private String idMessage;

	@Column(name="runtime_warning")
	private Boolean runtimeWarning;

	@Column(name="transition_end")
	private Timestamp transitionEnd;

	@Column(name="transition_execution_time")
	private BigInteger transitionExecutionTime;

	@Column(name="transition_start")
	private Timestamp transitionStart;

	//bi-directional many-to-one association to LogJournal
	@OneToMany(mappedBy="transitionInstance")
	private List<LogJournal> logJournals;

	//bi-directional many-to-one association to ProcessInstance
    @ManyToOne
	@JoinColumn(name="id_process_instance")
	private ProcessInstance processInstance;

	//bi-directional many-to-one association to Step
    @ManyToOne
	@JoinColumn(name="id_step_source")
	private Step step1;

	//bi-directional many-to-one association to Step
    @ManyToOne
	@JoinColumn(name="id_step_target")
	private Step step2;

	//bi-directional many-to-one association to TransitionState
    @ManyToOne
	@JoinColumn(name="id_state")
	private TransitionState transitionState;

    public TransitionInstance() {
    }

	public Long getIdJournal() {
		return this.idJournal;
	}

	public void setIdJournal(Long idJournal) {
		this.idJournal = idJournal;
	}

	public String getAttachmentId() {
		return this.attachmentId;
	}

	public void setAttachmentId(String attachmentId) {
		this.attachmentId = attachmentId;
	}

	public int getAttachmentIn() {
		return this.attachmentIn;
	}

	public void setAttachmentIn(int attachmentIn) {
		this.attachmentIn = attachmentIn;
	}

	public BigInteger getIdExtProcessInstance() {
		return this.idExtProcessInstance;
	}

	public void setIdExtProcessInstance(BigInteger idExtProcessInstance) {
		this.idExtProcessInstance = idExtProcessInstance;
	}

	public String getIdMessage() {
		return this.idMessage;
	}

	public void setIdMessage(String idMessage) {
		this.idMessage = idMessage;
	}

	public Boolean isRuntimeWarning() {
		return this.runtimeWarning;
	}

	public void setRuntimeWarning(Boolean runtimeWarning) {
		this.runtimeWarning = runtimeWarning;
	}

	public Timestamp getTransitionEnd() {
		return this.transitionEnd;
	}

	public void setTransitionEnd(Timestamp transitionEnd) {
		this.transitionEnd = transitionEnd;
	}

	public BigInteger getTransitionExecutionTime() {
		return this.transitionExecutionTime;
	}

	public void setTransitionExecutionTime(BigInteger transitionExecutionTime) {
		this.transitionExecutionTime = transitionExecutionTime;
	}

	public Timestamp getTransitionStart() {
		return this.transitionStart;
	}

	public void setTransitionStart(Timestamp transitionStart) {
		this.transitionStart = transitionStart;
	}

	public List<LogJournal> getLogJournals() {
		return this.logJournals;
	}

	public void setLogJournals(List<LogJournal> logJournals) {
		this.logJournals = logJournals;
	}
	
	public ProcessInstance getProcessInstance() {
		return this.processInstance;
	}

	public void setProcessInstance(ProcessInstance processInstance) {
		this.processInstance = processInstance;
	}
	
	public Step getStep1() {
		return this.step1;
	}

	public void setStep1(Step step1) {
		this.step1 = step1;
	}
	
	public Step getStep2() {
		return this.step2;
	}

	public void setStep2(Step step2) {
		this.step2 = step2;
	}
	
	public TransitionState getTransitionState() {
		return this.transitionState;
	}

	public void setTransitionState(TransitionState transitionState) {
		this.transitionState = transitionState;
	}
	
}