/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.nodemanager.hazelcast;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.ebpm.nodemanager.api.INodeManager;
import org.eclipse.ebpm.nodemanager.api.INodeManagerListener;

import com.hazelcast.config.Config;
import com.hazelcast.core.Cluster;
import com.hazelcast.core.Hazelcast;
import com.hazelcast.core.HazelcastInstance;
import com.hazelcast.core.Member;
import com.hazelcast.core.MembershipEvent;
import com.hazelcast.core.MembershipListener;

public class HazelCastNodeManager implements INodeManager, MembershipListener {

	private HazelcastInstance hazelCastInstance = null;
	private INodeManagerListener listener = null;
	
	@Override
	public void disconnet() {

		
	}

	@Override
	public void join(String cluster, String nodeId) {
		Config config = new Config(); // or new XmlConfigBuilder().build();
		config.getGroupConfig().setName(cluster);
		hazelCastInstance = Hazelcast.newHazelcastInstance(config);
		
		Cluster hzCluster = hazelCastInstance.getCluster();
		
		Map<Member, String> memberIds = Hazelcast.getMap("member-ids"); 
		memberIds.put(hzCluster.getLocalMember(), nodeId);
		
		hazelCastInstance.getCluster().addMembershipListener(this);
		
		
		
		
	}

	@Override
	public void registerListener(INodeManagerListener nodeManagerList) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void memberAdded(MembershipEvent event) {
		notifyChange();
        
	}

	@Override
	public void memberRemoved(MembershipEvent event) {
		notifyChange();
		
	}


	protected void notifyChange(){
		Map<Member, String> memberIds = Hazelcast.getMap("member-ids"); 
		
		List<String> viewMembersAsString = new ArrayList<String>();
		for (Member member : hazelCastInstance.getCluster().getMembers()){
			String newViewNode = memberIds.get(member);
        	viewMembersAsString.add(newViewNode);
		}
		
        this.listener.onClusterChange(viewMembersAsString);
	}
	
}
