/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.api;

import java.util.Set;


public interface Interceptor {
    
    /**
     * Intercepts a message. 
     * Interceptors need NOT invoke handleMessage or handleFault
     * on the next interceptor - the interceptor chain will
     * take care of this.
     * 
     * @param message
     */
    void handleMessage(Message message);
    
    /**
     * Called for all interceptors (in reverse order) on which handleMessage
     * had been successfully invoked, when normal execution of the chain was
     * aborted for some reason.
     * 
     * @param message
     */
    void handleFault(Message message);
    
    /**
     * A Set of IDs that this interceptor needs to run after.
     * @return
     */
    Set<String> getAfter();

    /**
     * A Set of IDs that this interceptor needs to run before.
     * @return
     */
    Set<String> getBefore();

    /**
     * The ID of the interceptor.
     * @return
     */
    String getId();

}
