/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.bindings.http.impl;

import java.util.List;

import org.eclipse.ebpm.soap.api.Interceptor;
import org.eclipse.ebpm.soap.bindings.http.interceptors.HttpDecoderInterceptor;
import org.eclipse.ebpm.soap.bindings.http.interceptors.HttpInOperationInterceptor;
import org.eclipse.ebpm.soap.bindings.http.model.Wsdl2HttpBinding;
import org.eclipse.ebpm.soap.bindings.http.model.Wsdl2HttpOperation;
import org.eclipse.ebpm.soap.core.model.AbstractBinding;
import org.eclipse.ebpm.soap.interceptors.mime.AttachmentsOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.nmr.NmrFaultOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.nmr.NmrInInterceptor;
import org.eclipse.ebpm.soap.interceptors.nmr.NmrOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.BodyOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.StaxOutInterceptor;

public class Wsdl2HttpBindingImpl extends AbstractBinding<Wsdl2HttpOperation> implements Wsdl2HttpBinding {

    private String httpAuthenticationRealm;
    private AuthenticationType httpAuthenticationType;
    private boolean httpCookies;
    private String httpMethodDefault;
    private String httpQueryParameterSeparatorDefault;
    private String httpTransferCodingDefault;
    private String httpVersion;
    
    public Wsdl2HttpBindingImpl() {
        List<Interceptor> phase;
        
        // ServerIn phase
        phase = getInterceptors(Phase.ServerIn);
        phase.add(new HttpInOperationInterceptor());
        phase.add(new HttpDecoderInterceptor(true));
        phase.add(new NmrInInterceptor(true));
        
        // ServerOut phase
        phase = getInterceptors(Phase.ServerOut);
        phase.add(new NmrFaultOutInterceptor());
        phase.add(new NmrOutInterceptor(true));
        phase.add(new AttachmentsOutInterceptor());
        phase.add(new StaxOutInterceptor());
        phase.add(new BodyOutInterceptor());
        
        // ClientOut phase
        phase = getInterceptors(Phase.ClientOut);

        // ClientIn phase
        phase = getInterceptors(Phase.ClientIn);
    }
    
    /**
     * @return the httpAuthenticationRealm
     */
    public String getHttpAuthenticationRealm() {
        return httpAuthenticationRealm;
    }
    /**
     * @param httpAuthenticationRealm the httpAuthenticationRealm to set
     */
    public void setHttpAuthenticationRealm(String httpAuthenticationRealm) {
        this.httpAuthenticationRealm = httpAuthenticationRealm;
    }
    /**
     * @return the httpAuthenticationType
     */
    public AuthenticationType getHttpAuthenticationType() {
        return httpAuthenticationType;
    }
    /**
     * @param httpAuthenticationType the httpAuthenticationType to set
     */
    public void setHttpAuthenticationType(AuthenticationType httpAuthenticationType) {
        this.httpAuthenticationType = httpAuthenticationType;
    }
    /**
     * @return the httpCookies
     */
    public boolean isHttpCookies() {
        return httpCookies;
    }
    /**
     * @param httpCookies the httpCookies to set
     */
    public void setHttpCookies(boolean httpCookies) {
        this.httpCookies = httpCookies;
    }
    /**
     * @return the httpMethodDefault
     */
    public String getHttpMethodDefault() {
        return httpMethodDefault;
    }
    /**
     * @param httpMethodDefault the httpMethodDefault to set
     */
    public void setHttpMethodDefault(String httpMethodDefault) {
        this.httpMethodDefault = httpMethodDefault;
    }
    /**
     * @return the httpQueryParamterSeparatorDefault
     */
    public String getHttpQueryParameterSeparatorDefault() {
        return httpQueryParameterSeparatorDefault;
    }
    /**
     * @param httpQueryParamterSeparatorDefault the httpQueryParamterSeparatorDefault to set
     */
    public void setHttpQueryParameterSeparatorDefault(String httpQueryParamterSeparatorDefault) {
        this.httpQueryParameterSeparatorDefault = httpQueryParamterSeparatorDefault;
    }
    /**
     * @return the httpTransferCodingDefault
     */
    public String getHttpTransferCodingDefault() {
        return httpTransferCodingDefault;
    }
    /**
     * @param httpTransferCodingDefault the httpTransferCodingDefault to set
     */
    public void setHttpTransferCodingDefault(String httpTransferCodingDefault) {
        this.httpTransferCodingDefault = httpTransferCodingDefault;
    }
    /**
     * @return the httpVersion
     */
    public String getHttpVersion() {
        return httpVersion;
    }
    /**
     * @param httpVersion the httpVersion to set
     */
    public void setHttpVersion(String httpVersion) {
        this.httpVersion = httpVersion;
    }
}
