/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.bindings.soap.impl;

import java.util.List;

import org.eclipse.ebpm.soap.api.Interceptor;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.bindings.soap.SoapVersion;
import org.eclipse.ebpm.soap.bindings.soap.interceptors.MustUnderstandInterceptor;
import org.eclipse.ebpm.soap.bindings.soap.interceptors.SoapActionInOperationInterceptor;
import org.eclipse.ebpm.soap.bindings.soap.interceptors.SoapActionOutOperationInterceptor;
import org.eclipse.ebpm.soap.bindings.soap.interceptors.SoapDOM4JOutInterceptor;
import org.eclipse.ebpm.soap.bindings.soap.interceptors.SoapFaultInInterceptor;
import org.eclipse.ebpm.soap.bindings.soap.interceptors.SoapFaultOutInterceptor;
import org.eclipse.ebpm.soap.bindings.soap.interceptors.SoapInInterceptor;
import org.eclipse.ebpm.soap.bindings.soap.interceptors.SoapOutInterceptor;
import org.eclipse.ebpm.soap.bindings.soap.model.wsdl1.Wsdl1SoapBinding;
import org.eclipse.ebpm.soap.bindings.soap.model.wsdl1.Wsdl1SoapOperation;
import org.eclipse.ebpm.soap.core.model.AbstractBinding;
import org.eclipse.ebpm.soap.interceptors.mime.AttachmentsInInterceptor;
import org.eclipse.ebpm.soap.interceptors.mime.AttachmentsOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.nmr.NmrFaultOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.nmr.NmrInInterceptor;
import org.eclipse.ebpm.soap.interceptors.nmr.NmrOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.nmr.NmrWSDL1Interceptor;
import org.eclipse.ebpm.soap.interceptors.wsdl.WsdlOperationInInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.BodyOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.StaxInInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.StaxOutInterceptor;


public class Wsdl1SoapBindingImpl extends AbstractBinding<Wsdl1SoapOperation> implements Wsdl1SoapBinding {

    private SoapVersion soapVersion;
    private String locationURI;
    private String transportURI;
    private Style style;

    public Wsdl1SoapBindingImpl() {
        this(null);
    }
    
    public Wsdl1SoapBindingImpl(SoapVersion soapVersion) {
        this.soapVersion = soapVersion;
        
        List<Interceptor> phase;
        
        // ServerIn phase
        phase = getInterceptors(Phase.ServerIn);
        phase.add(new AttachmentsInInterceptor());
        phase.add(new StaxInInterceptor());
        phase.add(new SoapInInterceptor(soapVersion));
        phase.add(new SoapActionInOperationInterceptor());
        phase.add(new WsdlOperationInInterceptor());
        phase.add(new MustUnderstandInterceptor());
        phase.add(new NmrWSDL1Interceptor(true));
        phase.add(new NmrInInterceptor(true));
        

        // ServerOut phase
        phase = getInterceptors(Phase.ServerOut);
        //phase.add(new StaxOutInterceptor());
        phase.add(new NmrFaultOutInterceptor());
        phase.add(new NmrOutInterceptor(true));
        phase.add(new AttachmentsOutInterceptor());
        //phase.add(new SoapDOM4JOutInterceptor(soapVersion));
        phase.add(new StaxOutInterceptor());
        phase.add(new SoapOutInterceptor(soapVersion));
        phase.add(new BodyOutInterceptor());
        
        
        
        // ServerOutFault phase
        phase = getInterceptors(Phase.ServerOutFault);
//        phase.add(new StaxOutInterceptor());
//        phase.add(new SoapOutInterceptor(soapVersion));
        phase.add(new SoapFaultOutInterceptor());
        
        // ClientOut phase
        phase = getInterceptors(Phase.ClientOut);
        phase.add(new NmrOutInterceptor(false));
        phase.add(new SoapActionOutOperationInterceptor());
        phase.add(new AttachmentsOutInterceptor());
        phase.add(new SoapDOM4JOutInterceptor(soapVersion));
        /*
        phase.add(new StaxOutInterceptor());
        phase.add(new SoapOutInterceptor(soapVersion));
        phase.add(new BodyOutInterceptor());
		*/
        // ClientIn phase
        phase = getInterceptors(Phase.ClientIn);
        phase.add(new AttachmentsInInterceptor());
        phase.add(new StaxInInterceptor());
        phase.add(new SoapInInterceptor());
        phase.add(new SoapFaultInInterceptor());
        phase.add(new NmrWSDL1Interceptor(false));
        phase.add(new NmrInInterceptor(false));
    }

    /**
     * @return the locationURI
     */
    public String getLocationURI() {
        return locationURI;
    }

    /**
     * @param locationURI the locationURI to set
     */
    public void setLocationURI(String locationURI) {
        this.locationURI = locationURI;
    }

    /**
     * @return the transportURI
     */
    public String getTransportURI() {
        return transportURI;
    }

    /**
     * @param transportURI the transportURI to set
     */
    public void setTransportURI(String transportURI) {
        this.transportURI = transportURI;
    }

    /**
     * @return the style
     */
    public Style getStyle() {
        return style;
    }

    /**
     * @param style the style to set
     */
    public void setStyle(Style style) {
        this.style = style;
    }

    public Message createMessage() {
        Message msg = super.createMessage();
        if (msg.get(SoapVersion.class) == null && soapVersion != null) {
            msg.put(SoapVersion.class, soapVersion);
        }
        return msg;
    }
    
    public Message createMessage(Message request) {
        Message msg = super.createMessage(request);
        if (msg.get(SoapVersion.class) == null && request.get(SoapVersion.class) != null) {
            msg.put(SoapVersion.class, request.get(SoapVersion.class));
        }
        if (msg.get(SoapVersion.class) == null && soapVersion != null) {
            msg.put(SoapVersion.class, soapVersion);
        }
        return msg;
    }
    
}
