/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.bindings.soap.interceptors;

import java.io.OutputStream;
import java.net.URI;
import java.util.Iterator;

import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.io.DocumentSource;
import org.dom4j.io.XMLWriter;
import org.eclipse.ebpm.common.util.DOM4JUtils;
import org.eclipse.ebpm.soap.api.Fault;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.bindings.soap.Soap11;
import org.eclipse.ebpm.soap.bindings.soap.SoapConstants;
import org.eclipse.ebpm.soap.bindings.soap.SoapFault;
import org.eclipse.ebpm.soap.bindings.soap.SoapVersion;
import org.eclipse.ebpm.soap.core.AbstractInterceptor;

public class SoapFaultOutInterceptor extends AbstractInterceptor {

    public void handleMessage(Message message) {
    try{
    	Exception exception = message.getContent(Exception.class);
    	OutputStream os = message.getContent(OutputStream.class);
    	SoapVersion soapVersion = message.get(SoapVersion.class);
    	
        if (os == null) {
            throw new NullPointerException("XMLStreamWriter content not found");
        }
       
        XMLWriter writer = new XMLWriter(os);
    	Document doc = DocumentHelper.createDocument();
    	
    	Namespace soapNameSpace = new Namespace(soapVersion.getPrefix(), soapVersion.getNamespace());
    	
    	org.dom4j.QName soapEnvelope = new org.dom4j.QName(soapVersion.getEnvelope().getLocalPart(), soapNameSpace); 
    	org.dom4j.Element soapEnvelopeNode = doc.addElement(soapEnvelope);
        
        if (message.getSoapHeaders().size() > 0) {
            for (Iterator it = message.getSoapHeaders().values().iterator(); it.hasNext();) {
                //DocumentFragment df = (DocumentFragment) it.next();
                //Element e = DomUtil.getFirstChildElement(df);
            }
            
        }
        org.dom4j.QName soapBody = new org.dom4j.QName(soapVersion.getBody().getLocalPart(), soapNameSpace);
        org.dom4j.Element soapBodyNode =  soapEnvelopeNode.addElement(soapBody);
        
        Element faultElement = null;
        if (exception instanceof SoapFault) {
            SoapFault fault = (SoapFault) exception;
            if (soapVersion == null) {
                soapVersion = Soap11.getInstance();
            }
            if (soapVersion.getVersion() == 1.1) {
                faultElement = writeSoap11FaultDOM4J(os, fault, soapVersion,soapBodyNode);
            } else if (soapVersion.getVersion() == 1.2) {
            	faultElement = writeSoap12FaultDOM4J(os, fault, soapVersion,soapBodyNode );
            } else {
                throw new IllegalStateException("Unrecognized soap version: " + soapVersion.getVersion());
            }
            //soapBodyNode.add(faultElement);
        }
        
        System.out.println(doc.asXML());
        writer.write( doc );          
        writer.flush();

        
    } catch (Exception e) {
        throw new Fault(e);
    }
    }


	private Element writeSoap12FaultDOM4J(OutputStream os, SoapFault fault, SoapVersion soapVersion, Element bodyElement) throws Exception {
		fault.translateCodeTo12();
        
		Element faultElement = bodyElement.addElement(DOM4JUtils.toDOM4JQName(soapVersion.getFault()));
        QName code = fault.getCode();
        if (code != null) {
        	Element codeElement = faultElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_12_FAULTCODE));
	    	
            
	    	Element codeValueElement = codeElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_12_FAULTVALUE));
	    	codeValueElement.setText(soapVersion.getEnvelope().getLocalPart() + ":" + code.getLocalPart());
	    	
          
           
            QName subcode = fault.getSubcode();
            if (subcode != null) {
            	Element subcodeElement = codeElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_12_FAULTSUBCODE));
            	Element subcodeValueElement = subcodeElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_12_FAULTVALUE));
               
            	subcodeValueElement.setText(soapVersion.getEnvelope().getLocalPart() + ":" + subcode.getLocalPart());
            }
        }
        String reason = fault.getReason();
        if (reason == null && fault.getCause() != null) {
            reason = fault.getCause().toString();
        }
        Element reasonElement = faultElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_12_FAULTREASON));
        Element reasonFaultTextElement = reasonElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_12_FAULTTEXT));
        
       
        reasonFaultTextElement.addAttribute("lang", "en");
       
        if (reason != null) {
        	reasonFaultTextElement.setText(reason);
        }
       
        URI node = fault.getNode();
        
       
	    if (node != null) {
	    	Element faultActorElement = faultElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_12_FAULTNODE));
	    	faultActorElement.setText(node.toString());
	       
	    }
       

        URI role = fault.getRole();
        if (role != null) {
            Element faultRoleElement = faultElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_12_FAULTROLE));
            faultRoleElement.setText(node.toString());
        }

       
        Source details = fault.getDetails();
	    if (details != null) {
	    	
	    	
	    	Element faultDetailElement = faultElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_12_FAULTDETAIL));
	    	if (details instanceof DocumentSource){
	    		DocumentSource ds = (DocumentSource)details;
	    		Element detElem = ds.getDocument().getRootElement();
	    		detElem.detach();
	    		
	    		
	    		faultDetailElement.setText(detElem.asXML());
	    	}
	    }
	    return faultElement;

	}
	private Element writeSoap11FaultDOM4J(OutputStream os, SoapFault fault, SoapVersion soapVersion, Element bodyElement) throws Exception {
		fault.translateCodeTo11();
		
		Element faultElement = bodyElement.addElement(DOM4JUtils.toDOM4JQName(soapVersion.getFault()));
		    QName code = fault.getCode();
		    if (code != null) {
		    	//code.setPrefix(soapVersion.getEnvelope().getLocalPart());
		    	Element codeElement = faultElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_11_FAULTCODE));
		    	codeElement.setText(soapVersion.getEnvelope().getLocalPart() + ":" + code.getLocalPart());
		        
		    }
		    String reason = fault.getReason();
		    if (reason == null && fault.getCause() != null) {
		        reason = fault.getCause().toString();
		    }
		    
		    Element faultStringElement = faultElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_11_FAULTSTRING));
		    if (reason != null) {
		    	 faultStringElement.setText(reason);
		    }
		
		    URI node = fault.getNode();
		    if (node != null) {
		    	Element faultActorElement = faultElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_11_FAULTACTOR));
		    	faultActorElement.setText(node.toString());
		       
		    }
		    Source details = fault.getDetails();
		    if (details != null) {
		    	
		    	
		    	Element faultDetailElement = faultElement.addElement(DOM4JUtils.toDOM4JQName(SoapConstants.SOAP_11_FAULTDETAIL));
		    	if (details instanceof DocumentSource){
		    		DocumentSource ds = (DocumentSource)details;
		    		Element detElem = ds.getDocument().getRootElement();
		    		detElem.detach();
		    		
		    		
		    		faultDetailElement.setText(detElem.asXML());
		    	}
		    }
		    
		  return faultElement;
}
/*
   
public void handleMessage(Message message) {
        Exception exception = message.getContent(Exception.class);
        XMLStreamWriter writer = message.getContent(XMLStreamWriter.class);
        SoapVersion soapVersion = message.get(SoapVersion.class);
        try {
            if (exception instanceof SoapFault) {
                SoapFault fault = (SoapFault) exception;
                if (soapVersion == null) {
                    soapVersion = Soap11.getInstance();
                }
                if (soapVersion.getVersion() == 1.1) {
                    writeSoap11Fault(writer, fault, soapVersion);
                } else if (soapVersion.getVersion() == 1.2) {
                    writeSoap12Fault(writer, fault, soapVersion);
                } else {
                    throw new IllegalStateException("Unrecognized soap version: " + soapVersion.getVersion());
                }
            }
        } catch (XMLStreamException e) {
            throw new Fault(e);
        }
      
    }

    private void writeSoap11Fault(XMLStreamWriter writer, SoapFault fault, SoapVersion soapVersion) throws XMLStreamException {
        fault.translateCodeTo11();

        StaxUtil.writeStartElement(writer, soapVersion.getFault());
        QName code = fault.getCode();
        if (code != null) {
            StaxUtil.writeStartElement(writer, SoapConstants.SOAP_11_FAULTCODE);
            StaxUtil.writeTextQName(writer, code);
            writer.writeEndElement();
        }
        String reason = fault.getReason();
        if (reason == null && fault.getCause() != null) {
            reason = fault.getCause().toString();
        }
        StaxUtil.writeStartElement(writer, SoapConstants.SOAP_11_FAULTSTRING);
        if (reason != null) {
            writer.writeCharacters(reason);
        }
        writer.writeEndElement();
        URI node = fault.getNode();
        if (node != null) {
            StaxUtil.writeStartElement(writer, SoapConstants.SOAP_11_FAULTACTOR);
            writer.writeCharacters(node.toString());
            writer.writeEndElement();
        }
        Source details = fault.getDetails();
        if (details != null) {
            StaxUtil.writeStartElement(writer, SoapConstants.SOAP_11_FAULTDETAIL);
            XMLStreamReader reader = StaxUtil.createReader(details);
            StaxUtil.copy(reader, writer);
            writer.writeEndElement();
        }

        writer.writeEndElement();
    }

    private void writeSoap12Fault(XMLStreamWriter writer, SoapFault fault, SoapVersion soapVersion) throws XMLStreamException {
        fault.translateCodeTo12();
        
        StaxUtil.writeStartElement(writer, soapVersion.getFault());
        QName code = fault.getCode();
        if (code != null) {
            StaxUtil.writeStartElement(writer, SoapConstants.SOAP_12_FAULTCODE);
            StaxUtil.writeStartElement(writer, SoapConstants.SOAP_12_FAULTVALUE);
            StaxUtil.writeTextQName(writer, code);
            writer.writeEndElement();
            QName subcode = fault.getSubcode();
            if (subcode != null) {
                StaxUtil.writeStartElement(writer, SoapConstants.SOAP_12_FAULTSUBCODE);
                StaxUtil.writeStartElement(writer, SoapConstants.SOAP_12_FAULTVALUE);
                StaxUtil.writeTextQName(writer, subcode);
                writer.writeEndElement();
                writer.writeEndElement();
            }
            writer.writeEndElement();
        }
        String reason = fault.getReason();
        if (reason == null && fault.getCause() != null) {
            reason = fault.getCause().toString();
        }
        StaxUtil.writeStartElement(writer, SoapConstants.SOAP_12_FAULTREASON);
        StaxUtil.writeStartElement(writer, SoapConstants.SOAP_12_FAULTTEXT);
        writer.writeAttribute(XMLConstants.XML_NS_PREFIX, XMLConstants.XML_NS_URI, "lang", "en");
        if (reason != null) {
            writer.writeCharacters(reason);
        }
        writer.writeEndElement();
        writer.writeEndElement();
        URI node = fault.getNode();
        if (node != null) {
            StaxUtil.writeStartElement(writer, SoapConstants.SOAP_12_FAULTNODE);
            writer.writeCharacters(node.toString());
            writer.writeEndElement();
        }

        URI role = fault.getRole();
        if (role != null) {
            StaxUtil.writeStartElement(writer, SoapConstants.SOAP_12_FAULTROLE);
            writer.writeCharacters(role.toString());
            writer.writeEndElement();
        }

        Source details = fault.getDetails();
        if (details != null) {
            StaxUtil.writeStartElement(writer, SoapConstants.SOAP_12_FAULTDETAIL);
            XMLStreamReader reader = StaxUtil.createReader(details);
            StaxUtil.copy(reader, writer);
            writer.writeEndElement();
        }

        writer.writeEndElement();
    }
    */
    
}
