/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.core.model;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;

import org.eclipse.ebpm.soap.api.InterceptorChain;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.api.model.Binding;
import org.eclipse.ebpm.soap.api.model.Operation;
import org.eclipse.ebpm.soap.core.AbstractInterceptorProvider;
import org.eclipse.ebpm.soap.core.MessageImpl;
import org.eclipse.ebpm.soap.core.PhaseInterceptorChain;


public class AbstractBinding<T extends Operation> extends AbstractInterceptorProvider 
                             implements Binding<T> {

    private QName interfaceName;
    private Map<QName, T> operations;
    private String location;
    
    public AbstractBinding() {
        operations = new HashMap<QName, T>();
    }
    
    public Message createMessage() {
        Message in = new MessageImpl();
        in.put(Binding.class, this);
        return in;
    }
    
    public Message createMessage(Message request) {
        Message out = new MessageImpl();
        out.put(Message.REQUEST_MESSAGE, request);
        out.put(Binding.class, this);
        Operation op = request.get(Operation.class);
        if (op != null) {
            out.put(Operation.class, op);
            out.put(org.eclipse.ebpm.soap.api.model.Message.class, op.getOutput());
        }
        return out;
    }
    
    public InterceptorChain getInterceptorChain(Phase phase) {
        InterceptorChain chain = new PhaseInterceptorChain();
        chain.add(getInterceptors(phase));
        return chain;
    }

    public QName getInterfaceName() {
        return interfaceName;
    }

    public void setInterfaceName(QName interfaceName) {
        this.interfaceName = interfaceName;
    }

    public T getOperation(QName name) {
        return operations.get(name);
    }

    public Collection<T> getOperations() {
        return operations.values();
    }
    
    public void addOperation(T operation) {
        operations.put(operation.getName(), operation);
    }

    /**
     * @return the location
     */
    public String getLocation() {
        return location;
    }

    /**
     * @param location the location to set
     */
    public void setLocation(String location) {
        this.location = location;
    }

}
