/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.interceptors.wsdl;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamReader;

import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.api.model.Binding;
import org.eclipse.ebpm.soap.api.model.Operation;
import org.eclipse.ebpm.soap.bindings.soap.SoapFault;
import org.eclipse.ebpm.soap.core.AbstractInterceptor;

/**

 */
public class WsdlOperationInInterceptor extends AbstractInterceptor {

    public void handleMessage(Message message) {
        if (message.get(Operation.class) != null) {
            return;
        }
        XMLStreamReader reader = message.getContent(XMLStreamReader.class);
        if (reader == null || reader.getEventType() != XMLStreamConstants.START_ELEMENT) {
            return;
        }
        Binding<?> binding = message.get(Binding.class);
        List<Operation> matching = new ArrayList<Operation>();
        QName name = reader.getName();
        for (Operation<?> oper : binding.getOperations()) {
//        	System.out.println("WSDL OperationInInterceptor --> Name is ["+name+"]");
//        	System.out.println("WSDL OperationInInterceptor --> Operation Input Name ["+oper.getInput().getElementName()+"]");
            if (name.equals(oper.getInput().getElementName())) {
                matching.add(oper);
            }
        }
        if (matching.size() == 1) {
            Operation op = matching.get(0);
            message.put(Operation.class, op);
            message.put(org.eclipse.ebpm.soap.api.model.Message.class, op.getInput());
        } else {
            StringBuilder sb = new StringBuilder();
            sb.append("No matching operation found for element: ");
            sb.append(name);
            sb.append("\n");
            sb.append("Expected one of: \n");
            for (Operation<?> oper : binding.getOperations()) {
                sb.append("  - ");
                sb.append(oper.getInput().getElementName());
                sb.append("\n");
            }
            throw new SoapFault(SoapFault.SENDER, sb.toString());
        }
    }

}
