/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.util.stax;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.NamespaceContext;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.util.StreamReaderDelegate;

public class ExtendedXMLStreamReader extends StreamReaderDelegate {

	private SimpleNamespaceContext context = new SimpleNamespaceContext();
	
	public ExtendedXMLStreamReader(XMLStreamReader delegate) {
		super(delegate);
	}

	public NamespaceContext getNamespaceContext() {
		return context;
	}
	
    public int nextTag() throws XMLStreamException {
    	int eventType = next();
    	 while((eventType == XMLStreamConstants.CHARACTERS && isWhiteSpace()) // skip whitespace
	    	 || (eventType == XMLStreamConstants.CDATA && isWhiteSpace()) 
	    	 // skip whitespace
	    	 || eventType == XMLStreamConstants.SPACE
	    	 || eventType == XMLStreamConstants.PROCESSING_INSTRUCTION
	    	 || eventType == XMLStreamConstants.COMMENT
	    	 ) {
	    	 eventType = next();
    	 }
    	 if (eventType != XMLStreamConstants.START_ELEMENT && eventType != XMLStreamConstants.END_ELEMENT) {
    	 	throw new XMLStreamException("expected start or end tag", getLocation());
    	 }
    	 return eventType;
    }

	public int next() throws XMLStreamException {
		int next = super.next();
		if (next == START_ELEMENT) {
			context = new SimpleNamespaceContext(context, getNamespaces());
		} else if (next == END_ELEMENT) {
			context = context.getParent();
		}
		return next;
	}

	private Map<String, String> getNamespaces() {
		Map<String,String> ns = new HashMap<String,String>();
		for (int i = 0; i < getNamespaceCount(); i++) {
			ns.put(getNamespacePrefix(i), getNamespaceURI(i));
		}
		return ns;
	}
	
	public static class SimpleNamespaceContext implements ExtendedNamespaceContext {

		private SimpleNamespaceContext parent;
		private Map<String,String> namespaces;
		
		public SimpleNamespaceContext() {
			namespaces = new HashMap<String,String>();
		}

		public SimpleNamespaceContext(SimpleNamespaceContext parent, Map<String,String> namespaces) {
			this.parent = parent;
			this.namespaces = namespaces;
		}

		public SimpleNamespaceContext getParent() {
			return parent;
		}
		
		public Collection<String> getPrefixes() {
			HashSet<String> prefixes = new HashSet<String>();
			for (SimpleNamespaceContext context = this; context != null; context = context.parent) {
				prefixes.addAll(context.namespaces.keySet());
			}
			return prefixes;
		}

		public String getNamespaceURI(String prefix) {
			String uri = namespaces.get(prefix);
			if (uri == null && parent != null) {
				uri = parent.getNamespaceURI(prefix);
			}
			return uri;
		}

		public String getPrefix(String namespaceURI) {
			for (SimpleNamespaceContext context = this; context != null; context = context.parent) {
                for (Map.Entry<String, String> entry : context.namespaces.entrySet()) {
					if (entry.getValue().equals(namespaceURI)) {
						return entry.getKey();
					}
				}
			}
			return null;
		}

		public Iterator<String> getPrefixes(String namespaceURI) {
			HashSet<String> prefixes = new HashSet<String>();
            for (SimpleNamespaceContext context = this; context != null; context = context.parent) {
                for (Map.Entry<String, String> entry : context.namespaces.entrySet()) {
                    if (entry.getValue().equals(namespaceURI)) {
						prefixes.add(entry.getKey());
					}
				}
			}
			return prefixes.iterator();
		}
		
	}


}
