/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.startup;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.startup.service.IStartedUpService;
import org.eclipse.ebpm.startup.service.SpagicStartedUpService;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;

public class ScrappyActivator implements BundleActivator {

	public void stop(BundleContext context) throws Exception {
	}

	protected void ebpmBanner(BundleContext context){
		BufferedReader reader = null;
		try{
			reader = new BufferedReader(new InputStreamReader(context.getBundle().getEntry("/baseconf/ebpm_banner.txt").openStream()));
			String line = null;
			while ((line = reader.readLine()) != null){
				System.out.println(line);
			}
		}catch(Exception e){
			e.printStackTrace();
		}finally{
			try{
				reader.close();
			}catch (IOException ioe) {
				ioe.printStackTrace();
			}
		}
	}
	protected void createFileFromBundleEntryIfNotExist(String filePath, String relativeBundleEntry, BundleContext context){

		File destFile = new File(filePath);
		if (!destFile.exists()){
			InputStream in = null;
			OutputStream out = null;
			try{
				destFile.createNewFile();
				in = context.getBundle().getEntry(relativeBundleEntry).openStream();
				out = new FileOutputStream(destFile);
				copy(in, out);
				out.flush();

			}catch(Exception e){
				e.printStackTrace();
			}finally{
				try{
					in.close();
					out.close();
				}catch (IOException ioe) {
					ioe.printStackTrace();
				}
			}
		}
	}

	
	protected String getDefaultNodeId(){

		return "node1";


	}

	protected void createSpagicXML(String filePath, BundleContext context){
		File destFile = new File(filePath);
		if (!destFile.exists()){
			XMLWriter w = null;
			FileWriter fw = null;
			try{
				OutputFormat format = OutputFormat.createPrettyPrint();
				fw = new FileWriter(destFile);
				w = new XMLWriter( fw , format );
				Document document = DocumentHelper.createDocument();
				Element root = document.addElement( "spagic" );
				root.addAttribute("nodeid", getDefaultNodeId());
				root.addAttribute("nodetype", "master");
				root.addAttribute("distributed", "false");

//				<cluster
//				clusterid="SpagicCluster"
//				masterslave="false|true"
//				referredmaster="mymaster">
//				distributedtopic="tcp://localhost:61616/exampleTopic"/>
				Element cluster = root.addElement("cluster");
				cluster.addAttribute("clusterid", "SpagicCluster");
				cluster.addAttribute("masterslave","false");
				cluster.addAttribute("referredmaster","mymaster");
				cluster.addAttribute("distributedtopic", "tcp://localhost:61616/exampleTopic");

				w.write( document );
			}catch(Exception e){
				e.printStackTrace();
			}finally{
				try{
					if (w != null){
						w.close();
					}else{
						if (fw != null)
							fw.close();
					}
				}catch (Exception e) {
					e.printStackTrace();
				}
			}

		}

	}

	protected void readNodeProperties(String spagicXML){

//		<?xml version="1.0" encoding="UTF-8"?>
//
//		<spagic nodeid="node1" 
//			nodetype="master"
//          distributed="true|false">

//			<cluster
//			clusterid="SpagicCluster"
//			masterslave="false|true"
//			referredmaster="mymaster">
//			distributedtopic="tcp://localhost:61616/exampleTopic"/>
//
//      </spagic> 
		try{
			
			System.out.println("Scrappy Startup --> Reading Node Properties");
			
			
			
			
			SAXReader reader = new SAXReader();
			Document doc = reader.read(new File(spagicXML));

			Element spagicElement = doc.getRootElement();
			
			String nodeId = spagicElement.attributeValue("nodeid");
			String nodeType = spagicElement.attributeValue("nodetype");
			String isDistributed = spagicElement.attributeValue("distributed");
			
			
			Element clusterElement = spagicElement.element("cluster");

			String clusterId = clusterElement.attributeValue("clusterid");
			String isMasterSlaveEnabled = clusterElement.attributeValue("masterslave");
			String referredMaster = clusterElement.attributeValue("referredmaster");
			
			System.out.println("Scrappy Startup --> Reading Node Properties NodeId["+nodeId+"]");
			System.out.println("Scrappy Startup --> Reading Node Properties NodeId["+nodeType+"]");
			System.out.println("Scrappy Startup --> Reading Node Properties isDistributed["+isDistributed+"]");
			System.out.println("Scrappy Startup --> Reading Node Properties clusterId["+clusterId+"]");
			System.out.println("Scrappy Startup --> Reading Node Properties isMasterSlaveEnabled["+isMasterSlaveEnabled+"]");
			System.out.println("Scrappy Startup --> Reading Node Properties referredMaster["+referredMaster+"]");



			String distributedEventAdmintopicUrl = clusterElement.attributeValue("distributedtopic");

			System.setProperty(SpagicConstants.SPAGIC_NODE_IDENTIFIER, nodeId);
			System.setProperty(SpagicConstants.SPAGIC_NODE_TYPE, nodeType);
			System.setProperty(SpagicConstants.DISTRIBUTED_EVENT_ADMIN_TOPIC_URL, distributedEventAdmintopicUrl);

			System.setProperty(SpagicConstants._CLUSTER_ID, clusterId);
			System.setProperty(SpagicConstants._IS_CLUSTER_ENABLED, isDistributed);
			System.setProperty(SpagicConstants._IS_MASTER_SLAVE_ENABLED,isMasterSlaveEnabled);
			System.setProperty(SpagicConstants._SLAVE_MASTER,referredMaster);



		}catch (Exception e) {
			e.printStackTrace();
		}

	}

	protected void nodeBootstrap(String home, BundleContext context) throws Exception {
		

		checkOrCreateFolder(home);

		String logBackFileLocation = System.getProperty(SpagicConstants.LOG_BACK_FILE_SYS_PROP);
		if (logBackFileLocation == null){
			logBackFileLocation = home + File.separator + "logback.xml";
			System.setProperty(SpagicConstants.LOG_BACK_FILE_SYS_PROP, logBackFileLocation);
		}
		createFileFromBundleEntryIfNotExist(logBackFileLocation, "/baseconf/logback.xml", context);


		String defaultProfileServiceFileLocation = home + File.separator + "userprofile.xml";
		createFileFromBundleEntryIfNotExist(defaultProfileServiceFileLocation, "/baseconf/userprofile.xml", context);
		
		String spagicXMLFileLocation = home + File.separator + "spagic.xml";
		createSpagicXML(spagicXMLFileLocation, context);
		
		
		String deployFileLocation = home + File.separator + "deploy.properties";
		createFileFromBundleEntryIfNotExist(deployFileLocation, "/baseconf/deploy.properties", context);


		String applicationFileLocation = home + File.separator + SpagicConstants.SPAGIC_APPLICATIONS_FILE_NAME;
		createFileFromBundleEntryIfNotExist(applicationFileLocation, "/baseconf/spagic-applications.xml", context);
		
		readNodeProperties(spagicXMLFileLocation);
	}

	protected void scrappyHomeBootstrap(String home, BundleContext context) throws Exception {
		String servicesFolder = home + File.separator + SpagicConstants.SERVICES_FOLDER;
		System.out.println("Scrappy Startup --> SERVICE FOLDER ["+servicesFolder+"]");
		checkOrCreateFolder(servicesFolder);


		String connectorFolder = home + File.separator + SpagicConstants.CONNECTORS_FOLDER;
		System.out.println("Scrappy Startup --> CONNECTOR FOLDER ["+connectorFolder+"]");
		checkOrCreateFolder(connectorFolder);

		String datasourcesFolder = home + File.separator + SpagicConstants.DATASOURCES_FOLDER;
		System.out.println("Scrappy Startup --> DATASOURCE FOLDER ["+datasourcesFolder+"]");
		checkOrCreateFolder(datasourcesFolder);

		String connectionFactoriesFolder = home + File.separator + SpagicConstants.CONNECTIONFACTORIES_FOLDER;
		System.out.println("Scrappy Startup --> CONNECTION FACTORIES FOLDER ["+connectionFactoriesFolder+"]");
		checkOrCreateFolder(connectionFactoriesFolder);

		String resourceFolder = home + File.separator + SpagicConstants.RESOURCES_FOLDER;
		System.out.println("Scrappy Startup --> RESOURCE FOLDER ["+resourceFolder+"]");
		checkOrCreateFolder(resourceFolder);

		String scriptFolder = resourceFolder + File.separator + "script";
		System.out.println("Scrappy Startup --> SCRIPT FOLDER ["+scriptFolder+"]");
		checkOrCreateFolder(scriptFolder);

		String wsdlFolder = resourceFolder + File.separator + "wsdl";
		System.out.println("Scrappy Startup --> WSDL FOLDER ["+wsdlFolder+"]");
		checkOrCreateFolder(wsdlFolder);

		String xsdFolder = resourceFolder + File.separator + "xsd";
		System.out.println("Scrappy Startup --> XSD FOLDER ["+xsdFolder+"]");
		checkOrCreateFolder(xsdFolder);

		String xslFolder = resourceFolder + File.separator + "xsd";
		System.out.println("Scrappy Startup --> XSL FOLDER ["+xslFolder+"]");
		checkOrCreateFolder(xslFolder);
		
		String propertiesFolder = resourceFolder + File.separator + "properties";
		System.out.println("Scrappy Startup --> SCRIPT FOLDER ["+propertiesFolder+"]");
		checkOrCreateFolder(propertiesFolder);

		String routesFolder = home + File.separator + SpagicConstants.ROUTES_FOLDER;
		System.out.println("Scrappy Startup --> ROUTES FOLDER ["+routesFolder+"]");
		checkOrCreateFolder(routesFolder);


		String jbpmDS = datasourcesFolder + File.separator + "jbpm.ds";
		createFileFromBundleEntryIfNotExist(jbpmDS, "/baseconf/jbpm.ds", context);

		String metadbDS = datasourcesFolder + File.separator + "metadb.ds";
		createFileFromBundleEntryIfNotExist(metadbDS, "/baseconf/metadb.ds", context);

		// Saint - Tue Feb 16 17:11:15 2010
		// 
		// Added the attachment store db (maybe is used, maybe not -
		// only the end user knows :)
		String attachmentDS = datasourcesFolder + File.separator + "attachments.ds";
		createFileFromBundleEntryIfNotExist(attachmentDS, "/baseconf/attachments.ds", context);
		//  </Saint>

		String deployResourceWSDLFile = wsdlFolder +  File.separator + "DeployResources.wsdl";
		createFileFromBundleEntryIfNotExist(deployResourceWSDLFile, "/baseconf/DeployResources.wsdl", context);

		String getResourceWSDLFile = wsdlFolder +  File.separator + "GetResources.wsdl";
		createFileFromBundleEntryIfNotExist(getResourceWSDLFile, "/baseconf/GetResources.wsdl", context);

		String jbpmEngineWSDLFile = wsdlFolder +  File.separator + "JBPMEngine.wsdl";
		createFileFromBundleEntryIfNotExist(jbpmEngineWSDLFile, "/baseconf/JBPMEngine.wsdl", context);

		// Saint - Fri Mar  5 15:46:49 2010
		// 
		// Changes for the service status inspection
		String serviCeconnectorsWSDLFile = wsdlFolder +  File.separator + "serviceconnectors.wsdl";
		createFileFromBundleEntryIfNotExist(serviCeconnectorsWSDLFile, "/baseconf/serviceconnectors.wsdl", context);

		String serviCeconnectorsToggleWSDLFile = wsdlFolder +  File.separator + "serviceconnectorstoggle.wsdl";
		createFileFromBundleEntryIfNotExist(serviCeconnectorsToggleWSDLFile, "/baseconf/serviceconnectorstoggle.wsdl", context);
		//  </Saint>


		String deployResourceConnectorFile = connectorFolder +  File.separator + "DeployResources.connector";
		createFileFromBundleEntryIfNotExist(deployResourceConnectorFile, "/baseconf/DeployResources.connector", context);

		String getResourceConnectorFile = connectorFolder +  File.separator + "GetResources.connector";
		createFileFromBundleEntryIfNotExist(getResourceConnectorFile, "/baseconf/GetResources.connector", context);

		String wfEngineWorkflowFile = connectorFolder +  File.separator + "WFEngine.connector";
		createFileFromBundleEntryIfNotExist(wfEngineWorkflowFile, "/baseconf/WFEngine.connector", context);

		// Saint - Fri Mar  5 15:47:43 2010
		// 
		// Changes for the service status inspection
		String connectorServiceConnectorStatus = connectorFolder +  File.separator + "ServicesConnectorStatus.connector";
		createFileFromBundleEntryIfNotExist(connectorServiceConnectorStatus, "/baseconf/ServicesConnectorStatus.connector", context);

		String connectorServiceConnectorToggle = connectorFolder +  File.separator + "ServicesConnectorToggle.connector";
		createFileFromBundleEntryIfNotExist(connectorServiceConnectorToggle, "/baseconf/ServicesConnectorToggle.connector", context);
		//  </Saint>

		String bpmServiceFile = servicesFolder +  File.separator + "SpagicBPMService.service";
		createFileFromBundleEntryIfNotExist(bpmServiceFile, "/baseconf/SpagicBPMService.service", context);

		String deployResourceServiceFile = servicesFolder +  File.separator + "SpagicDeployResource.service";
		createFileFromBundleEntryIfNotExist(deployResourceServiceFile, "/baseconf/SpagicDeployResource.service", context);

		String getResourceServiceFile = servicesFolder +  File.separator + "SpagicGetResource.service";
		createFileFromBundleEntryIfNotExist(getResourceServiceFile, "/baseconf/SpagicGetResource.service", context);

		String invokerServiceFile = servicesFolder +  File.separator + "SpagicInvoker.service";
		createFileFromBundleEntryIfNotExist(invokerServiceFile, "/baseconf/SpagicInvoker.service", context);

		// Saint - Fri Mar  5 15:48:17 2010
		// 
		// Changes for the service status inspection
		String serviceServiceConnectorStatus = servicesFolder +  File.separator + "ServicesConnectorStatus.service";
		createFileFromBundleEntryIfNotExist(serviceServiceConnectorStatus, "/baseconf/ServiceConnectorStatus.service", context);

		String serviceServiceConnectorToggle = servicesFolder +  File.separator + "ServicesConnectorToggle.service";
		createFileFromBundleEntryIfNotExist(serviceServiceConnectorToggle, "/baseconf/ServicesConnectorToggle.service", context);
		//  </Saint>

		//
		// Configuration For ClientConnectorProxy
		//
		String clientConnectorProxyFile = connectorFolder +  File.separator + "ClientProxyConnector.connector";
		createFileFromBundleEntryIfNotExist(clientConnectorProxyFile, "/baseconf/ClientProxyConnector.connector", context);
		
		String clientConnectorProxyFileWSDLFile = wsdlFolder +  File.separator + "ClientProxyConnectorConnector.wsdl";
		createFileFromBundleEntryIfNotExist(clientConnectorProxyFileWSDLFile, "/baseconf/ClientProxyConnectorConnector.wsdl", context);
		
	}

	public void start(BundleContext context) throws Exception {
		ebpmBanner(context);
		debugSystemProperties();

		String home = System.getProperty(SpagicConstants.SPAGIC_HOME_PROPERTY);
		if (home == null){
			home = System.getProperty("user.dir");
			home = home + File.separator + "default";
			System.out.println("Scrappy Startup --> SPAGIC_HOME NOT SET USE THE CURRENT FOLDER ["+home+"]");
			System.setProperty(SpagicConstants.SPAGIC_HOME_PROPERTY, home);
		}
		nodeBootstrap(home, context);
		scrappyHomeBootstrap(home, context);
		
		context.registerService(IStartedUpService.class.getName(), new SpagicStartedUpService(), null);
	}

	private void checkOrCreateFolder(String fName) throws Exception {
		File f = new File(fName + File.separator);
		if (!f.exists())
			f.mkdirs();

	}


	public void debugSystemProperties(){
		System.out.println("[=========================================================================]");
		System.out.println("[                      ENVIRONMENT INFO                                  =]"); 
		System.out.println("[=========================================================================]");
		Set<String> sysPropertiesNames = (Set<String>) System.getProperties().stringPropertyNames();


		List<String> pNames = new ArrayList<String>();
		pNames.addAll(sysPropertiesNames);

		Collections.sort(pNames);

		for (String name : pNames){
			System.out.println("["+name+"] -> ["+System.getProperty(name)+"]");
		}

	}

	void copy(InputStream in , OutputStream out) throws IOException {
        
        // Transfer bytes from in to out
        byte[] buf = new byte[1024];
        int len;
        while ((len = in.read(buf)) > 0) {
            out.write(buf, 0, len);
        }
       
    }

}
