/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.workflow.api;

/**
 * Extended Workflow API
 * To use only in case of emergency
 *
 */
public interface IControlExtendedAPI {

	/**
	 * Checks if two nodes are on the same execution token, within the process with the specified process
	 * instance id
	 * @param processInstanceId Process instance id
	 * @param sourceTaskName Source task name
	 * @param destTaskName Destination task name
	 * @return True if the nodes are on the same execution token
	 */
	public boolean areNodesOnSameToken(long processInstanceId, String sourceTaskName, String destTaskName);

	/**
	 * Force the execution token on the node sourceTaskName, within the process identified by processInstanceId,
	 * to the task with name destTaskName.
	 * This is the method to use when the process instance has multiple tokens.
	 * @param processInstanceId Process instance id
	 * @param sourceTaskName Current task name
	 * @param destTaskName Destination task name
	 */
	public void forceToken(long processInstanceId, String sourceTaskName, String destTaskName);
	
	/**
	 * Force the unique execution token within the process identified by processInstanceId to
	 * the task with name destTaskName.
	 * @param processInstanceId Process instance id
	 * @param destTaskName Destination task name
	 */
	public void forceToken(long processInstanceId, String destTaskName);
	
}
