/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.workflow.api;

import java.util.List;
import java.util.Map;

import org.eclipse.ebpm.metadata.SchemaDefinition;

public class Task {

	public static final int NO_FORM = 0;
	public static final int FORM_DYNAMIC = 1;
	public static final int FORM_FLEX = 2;
	public static final int FORM_ORBEON = 3;
	public static final int FORM_PDF = 4;
	
	public static final String NO_FORM_LBL = "NoForm";
	public static final String FORM_DYNAMIC_LBL = "dynamicForm";
	public static final String FORM_FLEX_LBL = "flexForm";
	public static final String FORM_ORBEON_LBL = "OrbeonForm";
	public static final String FORM_PDF_LBL = "PdfForm";

	private long taskId;
	private long internalId;
	private String taskName;

	private String processName;
	private long processId;

	private int formType;
	private int schemaIdentifier;
	private SchemaDefinition schemaDefinition;
	private String formTitle;
	
	// New properties to handle orbeon form identification
	private String formApplication;
	private String formForm;
	
	// task owning the form we want a PDF rendering of
	private String formTask;
	
	//map of the attributes of the Task
	private Map<String, String> taskAttributes;
	
	public String getFormTask() {
		return formTask;
	}

	public void setFormTask(String formTask) {
		this.formTask = formTask;
	}

	public String getFormApplication() {
		return formApplication;
	}

	public void setFormApplication(String formApplication) {
		this.formApplication = formApplication;
	}

	public String getFormForm() {
		return formForm;
	}

	public void setFormForm(String formForm) {
		this.formForm = formForm;
	}


	private List<String> enabledRoles = null;

	public List<String> getEnabledRoles() {
		return enabledRoles;
	}

	public void setEnabledRoles(List<String> enabledRoles) {
		this.enabledRoles = enabledRoles;
	}

	public long getTaskId() {
		return taskId;
	}

	public void setTaskId(long taskId) {
		this.taskId = taskId;
	}

	public String getTaskName() {
		return taskName;
	}

	public void setTaskName(String taskName) {
		this.taskName = taskName;
	}

	public long getInternalId() {
		return internalId;
	}

	public void setInternalId(long internalId) {
		this.internalId = internalId;
	}

	public int getFormType() {
		return formType;
	}

	public void setFormType(int formType) {
		this.formType = formType;
	}

	public int getSchemaIdentifier() {
		return schemaIdentifier;
	}

	public void setSchemaIdentifier(int schemaIdentifier) {
		this.schemaIdentifier = schemaIdentifier;
	}

	public SchemaDefinition getSchemaDefinition() {
		return schemaDefinition;
	}

	public void setSchemaDefinition(SchemaDefinition schemaDefinition) {
		this.schemaDefinition = schemaDefinition;
	}

	public String getProcessName() {
		return processName;
	}

	public void setProcessName(String processName) {
		this.processName = processName;
	}

	public long getProcessId() {
		return processId;
	}

	public void setProcessId(long processId) {
		this.processId = processId;
	}

	public void setFormTitle(String formTitle) {
		this.formTitle = formTitle;
	}

	public String getFormTitle() {
		return formTitle;
	}
	

	public Map<String, String> getTaskAttributes() {
		return taskAttributes;
	}

	public void setTaskAttributes(Map<String, String> taskAttributes) {
		this.taskAttributes = taskAttributes;
	}
	
	
	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (obj.getClass() != this.getClass()) {
			return false;
		}

		Task p = (Task) obj;
		if (this.taskId == p.taskId) {
			return true;
		}
		return false;
	}

}
