/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.xmlutil;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;

public class MapNamespaceContext implements NamespaceContext {
	private Map<String, String> namespaceMap;

	public MapNamespaceContext() {
		namespaceMap = new HashMap<String, String>();
		// namespaceMap.put("spagic", "urn:org:spagic3");
	}

	public MapNamespaceContext(Map<String, String> namespaceMap) {
		this.namespaceMap = namespaceMap;
	}

	public String getNamespaceURI(String prefix) {
		if (namespaceMap.containsKey(prefix))
			return namespaceMap.get(prefix);
		else
			return XMLConstants.NULL_NS_URI;
	}

	public String getPrefix(String namespace) {
		if (namespaceMap.containsValue(namespace)) {
			for (Entry<String, String> e : namespaceMap.entrySet()) {
				if (e.getValue().equals(namespace))
					return e.getKey();
			}
			return null;
		} else
			return null;
	}

	public Iterator<String> getPrefixes(String namespace) {
		return namespaceMap.keySet().iterator();
	}

}
