/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm.handler;

import org.eclipse.ebpm.core.ExchangeUtils;
import org.eclipse.ebpm.integration.api.IExchangeProvider;
import org.eclipse.ebpm.integration.api.IServiceRunner;
import org.eclipse.ebpm.integration.api.IWorkflowContextUpdater;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.jbpm.graph.def.ActionHandler;
import org.jbpm.graph.exe.ExecutionContext;
import org.spagic.workflow.logging.log.FaultLog;
import org.spagic3.components.bpm.BPMContextSingleton;

public class SpagicOSGiJBPMCustomHandler implements ActionHandler {

	private String serviceId;
	private String serviceRunnerClass;
	

	private String exchangeProviderClass;
	private String workflowContextUpdaterClass;
	
	
	
	public String getServiceId() {
		return serviceId;
	}

	public void setServiceId(String serviceId) {
		this.serviceId = serviceId;
	}

	
	
	public String getServiceRunnerClass() {
		return serviceRunnerClass;
	}

	public void setServiceRunnerClass(String serviceRunnerClass) {
		this.serviceRunnerClass = serviceRunnerClass;
	}
	
	public void execute(ExecutionContext ctx) throws Exception {
		try{
			IExchangeProvider exchangeProvider = getExchangeProvider();
			IServiceRunner runner = getServiceRunner();
		
			Exchange exchange = exchangeProvider.createExchangeFromWorkflowContext(ctx);
			

			exchange.setProperty(BPMContextSingleton.WORKFLOW_UPDATER_CLASS, workflowContextUpdaterClass);
			runner.run(serviceId, exchange);
		
			if (ExchangeUtils.isSync(exchange)){
				String workflowContextUpdaterClass = (String) exchange.getProperty(BPMContextSingleton.WORKFLOW_UPDATER_CLASS);
			
				IWorkflowContextUpdater updater = (IWorkflowContextUpdater)Class.forName(workflowContextUpdaterClass).newInstance();;
				updater.updateWorkflowContext(ctx, exchange);
			

			}
		} catch (Throwable t) {
			FaultLog faultLog = new FaultLog();
			faultLog.setToken(ctx.getToken());
			faultLog.setAction(ctx.getAction());
			faultLog.setException(t);
			faultLog.setProcessId(ctx.getProcessInstance().getId());
			faultLog.setProcessName(ctx.getProcessDefinition().getName());
			faultLog.setNodeName(ctx.getNode().getName());
			
			ctx.getProcessInstance().getRootToken().addLog(faultLog);

			
			ctx.getProcessInstance().end();
		} finally {
			
		}
	}
	
	public IServiceRunner getServiceRunner() throws Exception {
		return (IServiceRunner) Class.forName(serviceRunnerClass).newInstance();
	}
	
	public IExchangeProvider getExchangeProvider()  throws Exception {
		return (IExchangeProvider) Class.forName(exchangeProviderClass).newInstance();
	}
	
	public String getExchangeProviderClass() {
		return exchangeProviderClass;
	}

	public void setExchangeProviderClass(String exchangeProviderClass) {
		this.exchangeProviderClass = exchangeProviderClass;
	}

	public String getWorkflowContextUpdaterClass() {
		return workflowContextUpdaterClass;
	}

	public void setWorkflowContextUpdaterClass(String workflowContextUpdaterClass) {
		this.workflowContextUpdaterClass = workflowContextUpdaterClass;
	}
}
