/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm.invoker;

import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.core.AbstractSpagicService;
import org.eclipse.ebpm.core.ExchangeUtils;
import org.eclipse.ebpm.core.ISpagicService;
import org.eclipse.ebpm.integration.api.IWorkflowContextUpdater;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.components.bpm.BPMContextSingleton;
import org.spagic3.components.bpm.activator.BPMComponentActivator;

public class OSGiServiceInvoker extends AbstractSpagicService implements IServiceInvoker{

	private ConcurrentHashMap<String, Exchange> storedExchanges = new ConcurrentHashMap<String, Exchange>();
	private Logger logger = LoggerFactory.getLogger(OSGiServiceInvoker.class);
	
	
	
	@Override
	public void invokeService(String serviceID, Exchange exchange) {
		String exchangeId = exchange.getId();
		try{
			storedExchanges.put(exchangeId, exchange);
			logger.info(" Storing exchange ["+exchangeId+"] Associated to ["+this.getSpagicId()+"] -> ["+serviceID+"]");
			exchange.setProperty(SpagicConstants.SPAGIC_SENDER, this.getSpagicId());
			exchange.setProperty(SpagicConstants.SPAGIC_TARGET, serviceID);
			if (ExchangeUtils.isSync(exchange)){
				invokeSync(serviceID, exchange);
				process(exchange);
			}else{
				send(exchange);
			}
		}catch (Throwable e) {
			storedExchanges.remove(exchangeId);
			logger.error(e.getMessage(),e);
			throw new IllegalStateException(e);
		}
	}
	
	public void invokeSync(String serviceID, Exchange exchange) {
		ServiceReference[] refs = null;
		try {

			refs = BPMComponentActivator.getCtx().getServiceReferences(
					ISpagicService.class.getName(),
					"(" + SpagicConstants.SPAGIC_ID_PROPERTY + "=" + serviceID
							+ ")");
		} catch (InvalidSyntaxException e) {
			throw new IllegalStateException(e.getMessage(), e);
		}

		if (refs == null || (refs.length == 0))
			throw new IllegalStateException("Service with spagic.id ["
					+ serviceID + "] not found");
		if (refs != null && (refs.length > 1))
			throw new IllegalStateException(
					"Founded more than one service with the same spagicId ["
							+ serviceID + "]");

		ISpagicService service = null;

		if (refs[0] != null) {
			service = (ISpagicService) BPMComponentActivator.getCtx()
					.getService(refs[0]);

			if (service != null) {
				try {

					service.process(exchange);

				} finally {
					BPMComponentActivator.getCtx().ungetService(refs[0]);
				}
			} else {
				throw new IllegalStateException("Service is null");
			}
		} else {
			throw new IllegalStateException("Service Reference is null");
		}
	}

	@Override
	public void process(Exchange responseExchange) {
		try{
			logger.info(" ============= ["+ responseExchange.getProperty(SpagicConstants.SPAGIC_SENDER) + "] -> ["+responseExchange.getProperty(SpagicConstants.SPAGIC_TARGET) +"]");
			String id  = responseExchange.getId();
			logger.info("  Removing ["+responseExchange.getId()+"]");
			Exchange storedExchange = storedExchanges.remove(id);
			
			String tokenContext =(String) storedExchange.getProperty(BPMContextSingleton.TOKEN_CONTEXT);
			responseExchange.setProperty(BPMContextSingleton.TOKEN_CONTEXT, tokenContext);
			if (!ExchangeUtils.isSync(responseExchange)){
				String workflowContextUpdaterClass = (String) storedExchange.getProperty(BPMContextSingleton.WORKFLOW_UPDATER_CLASS);
			
				IWorkflowContextUpdater updater = (IWorkflowContextUpdater)Class.forName(workflowContextUpdaterClass).newInstance();;
				updater.updateWorkflowContext(null, responseExchange);
			}
			
		}catch (ClassNotFoundException cnfe) {
				throw new IllegalStateException(cnfe.getMessage(), cnfe);
		}catch (IllegalAccessException iae) {
				throw new IllegalStateException(iae.getMessage(), iae);
		}catch (InstantiationException ie) {
			throw new IllegalStateException(ie.getMessage(), ie);
		}
	} 
	
	
	
	

}
