/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.exchanges;

import org.eclipse.ebpm.messaging.api.Exchange;

/**
 * Interface <code>IExchangeStore</code> contract for the classes
 * providing persistence for the Exchange objects.
 *
 * @author <a href="mailto:saint@eng.it">Gian Uberto Lauri</a>
 * @version $Revision$
 */
public interface IExchangeStore {

	

	/**
	 * <code>store</code> persists an Exchange object
	 *
	 * @param pid a <code>Long</code> id of the Exchange object to be
	 * persisted
	 * @param ex an <code>Exchange</code> the Exchange object to be
	 * persisted
	 */
	public void store(Long pid, Exchange ex);

	/**
	 * <code>load</code> retrieves an Exchange object from its pid
	 *
	 * @param pid a <code>Long</code> id of the Exchange object to be
	 * retrieved
	 * @return an <code>Exchange</code> the Exchange object retrieved
	 */
	public Exchange load(Long pid);
	/**
	 * <code>delete</code> retrieves and deletes an Exchange object
	 * from its pid
	 *
	 * @param pid a <code>Long</code> id of the Exchange object to be
	 * retrieved and deleted to be persisted
	 * @return an <code>Exchange</code> the Exchange object retrieved
	 * and deleted to be persisted
	 */
	public Exchange delete(Long pid);

}
