package org.spagic3.jmsconnectionfactory.activemq;

import java.util.Properties;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.eclipse.ebpm.util.properties.PropertyConfigurator;
import org.jencks.amqpool.PooledConnectionFactory;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ConnectionFactory implements org.eclipse.ebpm.jms.connectionfactory.ConnectionFactory {
	
	private Logger logger = LoggerFactory.getLogger(ConnectionFactory.class);
	private PooledConnectionFactory pcf = null;
	private InitialContext initialCtx = null;
	
	@Override
	public javax.jms.Connection createConnection() throws JMSException {
		logger.debug("Creating connection");
		return pcf.createConnection();
	}

	@Override
	public javax.jms.Connection createConnection(String userName, String password)
			throws JMSException {
		logger.debug("Creating connection");
		return pcf.createConnection(userName, password);
	}

	protected void activate(ComponentContext componentContext){
		try{
			PropertyConfigurator propertyConfigurator = new PropertyConfigurator(componentContext.getProperties());
			String brokerURL = propertyConfigurator.getString("brokerURL");
			String factoryInitial = propertyConfigurator.getString("java.naming.factory.initial", "org.apache.activemq.jndi.ActiveMQInitialContextFactory");
			
			Properties props = new Properties();
			props.setProperty("java.naming.provider.url", brokerURL);
			props.setProperty("java.naming.factory.initial", factoryInitial);

			initialCtx = new InitialContext(props);			
			
			this.pcf = new PooledConnectionFactory(brokerURL);
		}catch (Exception e) {
			throw new IllegalStateException("Cannot istantiate ConnectionFactory", e);
		}
	}

	@Override
	public Destination getDestination(boolean pubSubDomain, String destinationName) throws NamingException {
		Destination result = (Destination)initialCtx.lookup((pubSubDomain ? "dynamicTopics/" : "dynamicQueues/") + destinationName);
		return result;
	}
	
}
